#########################################################################################
#########################################################################################
#########################################################################################
#											#
#											#
#											#																		 
#	R Code of Generalized Mixed Effects Regression Trees Function gmert()		#	
#											#
#		Ahlem HAJJEM*, Denis LAROCQUE and Franois BELLAVANCE			#
#											#
#				22th December 2016					#
#											#
#											#
#											#
#											#
# *Department of Marketing, Universit du Qubec  Montral,   				# 
#320, rue Sainte-Catherine Est, Montral (Qubec),            				#
#Canada, H2X 1L7.                                             				#
#hajjem.ahlem@uqam.ca                                         				#
#                                                             				#
# Department of Decision Sciences, HEC Montral,              				#
#3000 chemin de la Cte-Sainte-Catherine, Montral (Qubec),  				#
#Canada, H3T 2A7.                                             				#
#denis.larocque@hec.ca                                        				#
#                                                            				#
# Department of Decision Sciences, HEC Montral,              				#
#3000 chemin de la Cte-Sainte-Catherine, Montral (Qubec),  				#
#Canada, H3T 2A7. francois.bellavance@hec.ca                  				#
#                                                            				#
#########################################################################################
#########################################################################################
#########################################################################################


### IMPORTANT NOTE: ###
########################

# This document includes, successivley, : 

#a) A description of the arguments of gmert function
#b) R code of mert function (called within gmert function) 
#c) R code of gmert function


### End of Note ###											
###################




###########################################
###	Arguments of gmert Function	###
###########################################

#ni: a (1 X n) vector of clusters sizes in learning sample, where n is the number of clusters in learning sample.

#tni: a (1 X tn) vector of clusters sizes in test sample, where tn is the number of clusters in test sample.

#Xi: fixed-effects predictors from learning sample as a list of n matrices of dimensions (ni[i] X p), i= 1, ..., n, 
#where p is the number of fixed effects predictors.

#tXi: fixed-effects predictors from test sample as a list of tn matrices of dimensions (tni[i] X p), i= 1, ..., tn, 
#where p is the number of fixed effects predictors.

#Zi: random-effects predictors from learning sample as a list of n matrices of dimensions (ni[i] X q) , i= 1, ..., n, 
#where q is the number of random effects predictors (i.e. q = 1 in a random intercept model). 
#Note that the current implementation does not support q > 2.

#tZi: random-effects predictors from test sample as a list of tn matrices of dimensions (tni[i] X q) , i= 1, ..., tn, 
#where q is the number of random effects predictors (i.e. q = 1 in a random intercept model). 
#Note that the current implementation does not support q > 2. 

#Yi: responses from learning sample as a list of n matrices of dimensions (ni[i] X 1), i= 1, ..., n.

#tYi: responses from test sample as a list of tn matrices of dimensions (tni[i] X 1), i= 1, ..., tn.
 
#xnam: a (1 X p) character vector of names of fixed-effects predictors (same order as in Xi and tXi!).
		
#usesurrogate: various control parameters of rpart fits (see rpart.control function).
#surrogatestyle	
#maxsurrogate	
#maxcompete	
#minsplit	
#minbucket	
#maxdepth	
#xval		
#cp
		
#F: number of iterations forced at the start of the iteration process of mert function.	

#s.change: a small value choosen for mert function convergence (e.g. 1e-4). 

#mert.niter: max number of iterations allowed based on mert function convergence. 
#That is, mert function Keeps iterating while: 1) iterations number is less than (1 + F + mert.niter) and 
#2) absolute value of change in "GLL" is higher than s.change value.

#L: number of iterations forced at the end of the iteration process of mert function.	

#mert.verbose: logical. Should R reports extra information on progress about MERT fit?

#sigmasqzero: starting value of the dispersion parameter (e.g., sigmasqzero = 1). 

#Dzero: starting value of the covariance matrix of the random effects, e.g., Dzero = diag(x=0.01, nrow=q, ncol = q). 

#bizero: starting values of random effects as a list of n matrices of dimensions (q  1), e.g., bizero = matrix(0,nrow=q,ncol=1).
			
#family: gives the error distribution and link function (e.g. family=binomial).


#rc: a small value choosen for gmert function convergence (e.g. 1e-4). 

#gmert.niter: max number of macro-iterations allowed based on convergence. 
#That is, gmert function Keeps iterating while: 1) macro-iterations number is less than (1 + gmert.niter) and 
#2) relative change in "eta" is higher than or equals rc value.			

#S: number of macro-iterations forced at the end of the iteration process of gmert function.

#verbose: logical. Should R reports extra information on progress about GMERT fit?


# aggiunti da me, Chiara
# Dzero = diag(x=0.01, nrow=q, ncol = q)
# bizero = list()
# for(i in 1:50){
#   bizero[[i]] = matrix(0,nrow=q,ncol=1)
# }
# sigmasqzero = 1
# rc = 1e-4
# s.change= 1e-4
# mert.verbose=T
# verbose.gmert = T
# 
# usesurrogate=2
# surrogatestyle=0
# maxsurrogate=0
# maxcompete=2
# minsplit= 30	
# minbucket=10	
# maxdepth= 6	
# xval=10
# cp=0

######################################################################################################
######################################################################################################
###########################		mert Function		######################################
#####################	    (version called within gmert Function)	##############################
######################################################################################################
######################################################################################################

mert  <- function(
		 ni,Xi,Zi,Yi
		,tni,tXi,tZi,tYi
		,xnam

		,usesurrogate
		,surrogatestyle
		,maxsurrogate
		,maxcompete
		,minsplit
		,minbucket
		,maxdepth
		,xval=xval
		,cp=cp

		,F
		,s.change					
		,niter
		,L
		,verbose

		,sigmasqzero
		,Dzero
		,bizero
		,wi	
	)
{

	#########################################################
	####	STEP 0: Memory Allocation and initialization	#         
	#########################################################

	# values used in this function
		n <- length(ni)
		tn <- length(tni)
		N <- sum(ni)
		tN <- sum(tni)
		q <- dim(Zi[[1]])[2] 
		p <- dim(Xi[[1]])[2] 


	#iteration
		r <- 1

	#print micro iter number
		if (verbose) 	
			message("micro-iter ", r)

	#convergence criterion
		change <- vector(mode="numeric");length(change) <- r
		change[r] <- s.change

	#maximum number of iterations allowed, based on convergence criterion
		max.iter <- niter

	#weigheted data          
		wYi <- list();length(wYi) <- n
		wZi <- list();length(wZi) <- n
		for(i in 1:n){
			wYi[[i]] <- sqrt(wi[[i]])*Yi[[i]]	
			wZi[[i]] <- sqrt(wi[[i]])*Zi[[i]]
		}
 
	#transformed outcome
		star.Yi <- list() 
		for(i in 1:n){
			star.Yi[[i]] <- Yi[[i]] - Zi[[i]] %*% bizero[[i]]
		}

	#level-1 variance component
		sigma.sq <- vector(mode="numeric") ;length(sigma.sq) <- r
		sigma.sq[r] <- sigmasqzero	
		
	#level-2 variance component
		D <- list() ;length(D) <- r
		D[[r]] <- Dzero		
		
	#level-1 and level-2 variance components
		Vi <- list() 
		inv.Vi <- list(list()) ; length(inv.Vi) <- r

		for(i in 1:n){
			Vi[[i]] <- wZi[[i]] %*% D[[r]] %*% t(wZi[[i]])+sigma.sq[r]*diag(x = 1, nrow=ni[i], ncol = ni[i])
			inv.Vi[[r]][[i]] <- solve(Vi[[i]])
		}
		Vi <- list(NULL) 

	#use a standard weighted TREE model to predict fixed part component
	###################################################################

	#data in a data.frame format

		#learning
		gr<-list()
		for(i in 1:n)
			gr[[i]] <- rep(i, ni[i])
		mert.DBi <-list()
		for(i in 1:n)
			mert.DBi[[i]] <- as.data.frame(	
					cbind(	
						gr[[i]]
						,star.Yi[[i]]
						,Xi[[i]]
						,wi[[i]]
					)	
				)

		for(i in 2:n){
			mert.DBi[[i]] <- rbind(mert.DBi[[i-1]], mert.DBi[[i]])
			mert.DBi[i-1] <- list(NULL)
		}

		mert.DB <- as.data.frame(mert.DBi[[n]]) ; mert.DBi[n] <- list(NULL)
		names(mert.DB) <- c("gr","starY",xnam,"w")		 
		rm(mert.DBi);gc(verbose=FALSE)

		#test
		if(is.null(tni)==FALSE){
			gr<-list()
			for(i in 1:tn)
				gr[[i]] <- rep(i, tni[i])

				mert.tDBi <- list()
				for(i in 1:tn){
					mert.tDBi[[i]] <- as.data.frame(tXi[[i]])
					names(mert.tDBi[[i]]) <-  xnam
					
				}		
		}

		mert.formula <-  as.formula(paste("starY ~ ", paste(xnam, collapse= "+")))

	#large tree	
		tree <- rpart(
			formula = mert.formula
			,data= mert.DB
			,method="anova"
			,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete 
			,minsplit=minsplit,minbucket=minbucket ,maxdepth=maxdepth
			,xval=xval
			,cp=cp
			,weight=w
		)

		min.index <- 1
		if(dim(tree$cptable)[1]>1) min.index <- which.min(tree$cptable[,"xerror"])

	#pruned tree
		pruned.tree <- list(); length(pruned.tree)<-r
		pruned.tree[[r]] <- prune(tree, cp=tree$cptable[min.index,"CP"])	
		rm(tree, min.index);gc(verbose=FALSE)

	#nb of terminal nodes of the pruned tree
		pruned.tree.nb.leaves <- vector(mode="numeric") ; length(pruned.tree.nb.leaves) <- r
		pruned.tree.nb.leaves[r] <- dim(table(pruned.tree[[r]]$where))

	#fixed part predictions in matrix format
		#learning	
		mert.DBi <- list(); length(mert.DBi) <- n
		for(i in 1:n)
			mert.DBi[[i]] <- subset(mert.DB, gr==i)


		fixed.pred <- list(); length(fixed.pred) <- n

		fixed.pred[[1]] <- predict(pruned.tree[[r]],newdata= mert.DBi[[1]])

		for(i in 1:n){
			fixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.DBi[[i]])
			attr(fixed.pred[[i]], which="names") <- NULL
		}

		#test
		if(is.null(tni)==FALSE){
			tfixed.pred <- list()
			for(i in 1:tn){
				tfixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.tDBi[[i]])
				attr(tfixed.pred[[i]], which="names") <- NULL
			}


		}

	#random effects parameters
		bi <- list(list()) ; length(bi) <- r
		for(i in 1:n)bi[[r]][[i]] <- bizero[[i]] 	

	#total predictions
		#learning
		pred.Yi <- list(list());length(pred.Yi)<-r
		for(i in 1:n)
			pred.Yi[[r]][[i]] <- fixed.pred[[i]] + Zi[[i]] %*% bi[[r]][[i]]

		#test
		if(is.null(tni)==FALSE){
			tpred.Yi <- list(list());length(tpred.Yi) <- r	
			for(i in 1:tn){
				tpred.Yi[[r]][[i]] <- tfixed.pred[[i]] + tZi[[i]] %*% bi[[r]][[i]]
			}
		}


	#residuals
		epsili <- list(list());length(epsili)<-r 
		for(i in 1:n)
			epsili[[r]][[i]] <- wYi[[i]] - sqrt(wi[[i]])*pred.Yi[[r]][[i]]	

	#generalized log-likelihood (GLL) 
		GLL <- vector(mode="numeric") ; length(GLL) <- r
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i]<-t(epsili[[r]][[i]]) %*% solve(sigma.sq[r]*diag(x=1,nrow=ni[i],ncol=ni[i])) %*% epsili[[r]][[i]]
				+ t(bi[[r]][[i]]) %*% solve(D[[r]]) %*% bi[[r]][[i]]
				+ log(abs(D[[r]]))
				+ log(abs(sigma.sq[r]*diag(x=1, nrow=ni[i], ncol = ni[i])))
		GLL[r] <- sum(term)
		rm(term)
		gc(verbose=FALSE)

	#########################################################
	####	STEP 1	#######################################         
	#########################################################

	#update iteration number r
		r <- r+1
		if (verbose) 	
			message("micro-iter ", r)

	#update the length of the different lists
		length(pruned.tree)<-r
		length(pruned.tree.nb.leaves) <- r
		length(sigma.sq) <- r
		length(D) <- r
		length(inv.Vi) <- r
		length(pred.Yi)<- r	; pred.Yi[[r]]<-list()
		length(epsili) <- r	; epsili[[r]]<-list()
		length(bi) <- r
		length(GLL) <- r
		length(change) <- r
		if(is.null(tni)==FALSE){
			length(tpred.Yi) <- r	;tpred.Yi[[r]]<-list()
		}

	#update the transformed outcome, star.Yi
		for(i in 1:n){
			star.Yi[[i]] <- Yi[[i]] - Zi[[i]] %*% bi[[r-1]][[i]]	
		}	

	#use a standard weighted TREE model to predict fixed part component
	###################################################################
	#update starY values in mert.DB
		mert.DB$starY <- unlist(star.Yi)

	#large tree
		tree <- rpart(
			formula = mert.formula
			,data=mert.DB
			,method="anova"
			,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete 
			,minsplit=minsplit,minbucket=minbucket ,maxdepth=maxdepth
			,xval=xval
			,cp=cp
			,weight=w
		)

		min.index <- 1
		if(dim(tree$cptable)[1]>1) min.index <- which.min(tree$cptable[,"xerror"])

	#pruned tree
		pruned.tree[[r]] <- prune(tree, cp=tree$cptable[min.index,"CP"])
		pruned.tree[r-1] <- list(NULL)		
		rm(tree, min.index);gc(verbose=FALSE)

	#nb of terminal nodes of the pruned tree
		pruned.tree.nb.leaves[r] <- dim(table(pruned.tree[[r]]$where))

	#fixed part predictions in matrix format
		#learning	
		fixed.pred <- list()
		for(i in 1:n){
			fixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.DBi[[i]])
			attr(fixed.pred[[i]], which="names") <- NULL
		}

		#test
		if(is.null(tni)==FALSE){
			tfixed.pred <- list()
			for(i in 1:tn){
				tfixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.tDBi[[i]])
				attr(tfixed.pred[[i]], which="names") <- NULL
			}
		}
				
	#update the estimation of random effects parameters
		for(i in 1:n)
			bi[[r]][[i]] <- D[[r-1]]%*%t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]] %*% (wYi[[i]] - sqrt(wi[[i]])* fixed.pred[[i]])
		bi[r-1] <- list(NULL)		

	#########################################################
	####	STEP 2	#######################################         
	#########################################################

	#total predictions
		#learning
		for(i in 1:n)
			pred.Yi[[r]][[i]] <- fixed.pred[[i]] + Zi[[i]] %*% bi[[r]][[i]]	
		pred.Yi[r-1] <- list(NULL)

		#test
		if(is.null(tni)==FALSE){
			for(i in 1:tn)
				tpred.Yi[[r]][[i]] <- tfixed.pred[[i]] + tZi[[i]] %*% bi[[r]][[i]]
				tpred.Yi[r-1] <- list(NULL)
		}

	#residuals at current iteration r
		for(i in 1:n)
			epsili[[r]][[i]] <- wYi[[i]] - sqrt(wi[[i]])*pred.Yi[[r]][[i]]
		epsili[r-1] <- list(NULL)
			
	#update level-1 variance component
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- crossprod(epsili[[r]][[i]]) + 
           			sigma.sq[r-1] * (ni[i] - sigma.sq[r-1]* sum(diag(inv.Vi[[r-1]][[i]])))
		sigma.sq[r] <- (1/N)*(sum(term))
		rm(term)
		gc(verbose=FALSE)
		
	#update level-2 variance component
		term <- list()
		term[[1]] <- tcrossprod(bi[[r]][[1]]) + 
                           (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[1]])%*% inv.Vi[[r-1]][[1]] %*% wZi[[1]] %*% D[[r-1]]
                            )
		for(i in 2:n) 
			term[[i]] <- term[[i-1]]+ tcrossprod(bi[[r]][[i]]) + 
                        (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]]%*% wZi[[i]]%*% D[[r-1]]
                        )
		term <- term[[n]]
		D[[r]] <- (1/n)*term
		rm(term)
		gc(verbose=FALSE)

		D[[r-1]] <- list(NULL) 
	
	#level-1 and level-2 variance components at current iteration r
		inv.Vi[[r]] <-list()
		for(i in 1:n){
			Vi[[i]] <- wZi[[i]] %*% D[[r]] %*% t(wZi[[i]])+sigma.sq[r]*diag(x = 1, nrow=ni[i], ncol = ni[i])
			inv.Vi[[r]][[i]] <- solve(Vi[[i]])
		}
		Vi <- list(NULL) 
		inv.Vi[[r-1]] <- list(NULL) 	
  
	#Update GLL value
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- t(epsili[[r]][[i]]) %*% solve(sigma.sq[r]*diag(x=1,nrow=ni[i],ncol=ni[i])) %*% epsili[[r]][[i]]
				+ t(bi[[r]][[i]]) %*% solve(D[[r]]) %*% bi[[r]][[i]]
				+ log(abs(D[[r]]))
				+ log(abs(sigma.sq[r]*diag(x=1, nrow=ni[i], ncol = ni[i])))
		GLL[r] <- sum(term)
		rm(term)
		gc(verbose=FALSE)

	#update the value of the change in GLL
		change[r] <- abs((GLL[r]-GLL[r-1]))
		
	#########################################################
	####	STEP 3	#######################################         
	#########################################################

	#################################
	#	FIRST F ITERATIONS	#
	#################################

		ri <- r
		while( r < (ri+F) ){#repeat step 1 and 2

	#########################################################
	####	STEP 1	#######################################         
	#########################################################

	#update iteration number r
		r <- r+1
		if (verbose) 	
			message("micro-iter ", r)

	#update the length of the different lists
		length(pruned.tree)<-r
		length(pruned.tree.nb.leaves) <- r
		length(sigma.sq) <- r
		length(D) <- r
		length(inv.Vi) <- r
		length(pred.Yi)<- r	; pred.Yi[[r]]<-list()
		length(epsili) <- r	; epsili[[r]]<-list()
		length(bi) <- r
		length(GLL) <- r
		length(change) <- r
		if(is.null(tni)==FALSE){
			length(tpred.Yi) <- r	;tpred.Yi[[r]]<-list()
		}

	#update the transformed outcome, star.Yi
		for(i in 1:n){
			star.Yi[[i]] <- Yi[[i]] - Zi[[i]] %*% bi[[r-1]][[i]]	
		}	

	#use a standard weighted TREE model to predict the fixed part
	#############################################################
	#update starY values in mert.DB
		mert.DB$starY <- unlist(star.Yi)

	#large tree
		tree <- rpart(
			formula = mert.formula
			,data=mert.DB
			,method="anova"
			,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete 
			,minsplit=minsplit,minbucket=minbucket ,maxdepth=maxdepth
			,xval=xval
			,cp=cp
			,weight=w
		)
		min.index <- 1
		if(dim(tree$cptable)[1]>1) min.index <- which.min(tree$cptable[,"xerror"])

	#pruned tree
		pruned.tree[[r]] <- prune(tree, cp=tree$cptable[min.index,"CP"])		
		pruned.tree[r-1] <- list(NULL)		
		rm(tree, min.index);gc(verbose=FALSE)

	#nb of terminal nodes of the pruned tree
		pruned.tree.nb.leaves[r] <- dim(table(pruned.tree[[r]]$where))

	#fixed part predictions in matrix format
		#learning	
		fixed.pred <- list()
		for(i in 1:n){
			fixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.DBi[[i]])
			attr(fixed.pred[[i]], which="names") <- NULL
		}

		#test
		if(is.null(tni)==FALSE){
			tfixed.pred <- list()
			for(i in 1:tn){
				tfixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.tDBi[[i]])
				attr(tfixed.pred[[i]], which="names") <- NULL
			}
		}
				
	#update the estimation of random effects parameters
		for(i in 1:n)
			bi[[r]][[i]] <- D[[r-1]]%*%t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]] %*% (wYi[[i]] - sqrt(wi[[i]])* fixed.pred[[i]])
		bi[r-1] <- list(NULL)		
		
	#########################################################
	####	STEP 2	#######################################         
	#########################################################

	#total predictions
		#learning
		for(i in 1:n)
			pred.Yi[[r]][[i]] <- fixed.pred[[i]] + Zi[[i]] %*% bi[[r]][[i]]
			pred.Yi[r-1] <- list(NULL)

		#test
		if(is.null(tni)==FALSE){
			for(i in 1:tn)
				tpred.Yi[[r]][[i]] <- tfixed.pred[[i]] + tZi[[i]] %*% bi[[r]][[i]]
				tpred.Yi[r-1] <- list(NULL)
		}

	#residuals at current iteration r
		for(i in 1:n)
			epsili[[r]][[i]] <- wYi[[i]] - sqrt(wi[[i]])*pred.Yi[[r]][[i]]
			epsili[r-1] <- list(NULL)
			
	#update level-1 variance component
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- crossprod(epsili[[r]][[i]]) + 
           			sigma.sq[r-1] * (ni[i] - sigma.sq[r-1]* sum(diag(inv.Vi[[r-1]][[i]])))
		sigma.sq[r] <- (1/N)*(sum(term))
		rm(term)
		gc(verbose=FALSE)

	#update level-2 variance component
		term <- list()
		term[[1]] <- tcrossprod(bi[[r]][[1]]) + 
                           (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[1]])%*% inv.Vi[[r-1]][[1]] %*% wZi[[1]] %*% D[[r-1]]
                            )
		for(i in 2:n) 
			term[[i]] <- term[[i-1]]+ tcrossprod(bi[[r]][[i]]) + 
                        (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]]%*% wZi[[i]]%*% D[[r-1]]
                        )
		term <- term[[n]]
		D[[r]] <- (1/n)*term
		rm(term)
		gc(verbose=FALSE)
		D[[r-1]] <- list(NULL) 	
	
	#level-1 and level-2 variance components at current iteration r
		inv.Vi[[r]] <-list()
		for(i in 1:n){
			Vi[[i]] <- wZi[[i]] %*% D[[r]] %*% t(wZi[[i]])+sigma.sq[r]*diag(x = 1, nrow=ni[i], ncol = ni[i])
			inv.Vi[[r]][[i]] <- solve(Vi[[i]])
		}
		Vi <- list(NULL) 
		inv.Vi[[r-1]] <- list(NULL) 	
  
	#Update GLL value
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- t(epsili[[r]][[i]]) %*% solve(sigma.sq[r]*diag(x=1,nrow=ni[i],ncol=ni[i])) %*% epsili[[r]][[i]]
				+ t(bi[[r]][[i]]) %*% solve(D[[r]]) %*% bi[[r]][[i]]
				+ log(abs(D[[r]]))
				+ log(abs(sigma.sq[r]*diag(x=1, nrow=ni[i], ncol = ni[i])))
		GLL[r] <- sum(term)
		rm(term)
		gc(verbose=FALSE)

	#update the value of the change in GLL
		change[r] <- abs((GLL[r]-GLL[r-1]))

	#########################################
	}#	END OF FIRST F ITERATIONS	#	
	#########################################

	#########################################################
	#	ITERATIONS BASED ON CONVERGENCE CRITERION	#
	#########################################################

		while(r < (F+max.iter+1)){ 

			if(change[r] <= s.change)
				if (verbose) 
					message("micro-convergence at micro-iter ",r)
 
			if(change[r] > s.change){#repeat step 1 and 2

	#########################################################
	####	STEP 1	#######################################         
	#########################################################

	#update iteration number r
		r <- r+1
		if (verbose) 	
			message("micro-iter ", r)

	#update the length of the different lists
		length(pruned.tree)<-r
		length(pruned.tree.nb.leaves) <- r
		length(sigma.sq) <- r
		length(D) <- r
		length(inv.Vi) <- r
		length(pred.Yi)<- r	; pred.Yi[[r]]<-list()
		length(epsili) <- r	; epsili[[r]]<-list()
		length(bi) <- r
		length(GLL) <- r
		length(change) <- r
		if(is.null(tni)==FALSE){
			length(tpred.Yi) <- r	;tpred.Yi[[r]]<-list()
		}

	#update the transformed outcome, star.Yi
		for(i in 1:n){
			star.Yi[[i]] <- Yi[[i]] - Zi[[i]] %*% bi[[r-1]][[i]]	
		}	

	#use a standard weighted TREE model to predict the fixed part
	#############################################################
	#update starY values in mert.DB
		mert.DB$starY <- unlist(star.Yi)

	#large tree
		tree <- rpart(
			formula = mert.formula
			,data=mert.DB
			,method="anova"
			,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete 
			,minsplit=minsplit,minbucket=minbucket ,maxdepth=maxdepth
			,xval=xval
			,cp=cp
			,weight=w
		)
		min.index <- 1
		if(dim(tree$cptable)[1]>1) min.index <- which.min(tree$cptable[,"xerror"])

	#pruned tree
		pruned.tree[[r]] <- prune(tree, cp=tree$cptable[min.index,"CP"])
		pruned.tree[r-1] <- list(NULL)		
		rm(tree, min.index);gc(verbose=FALSE)

	#nb of terminal nodes of the pruned tree
		pruned.tree.nb.leaves[r] <- dim(table(pruned.tree[[r]]$where))

	#fixed part predictions in matrix format
		#learning	
		fixed.pred <- list()
		for(i in 1:n){
			fixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.DBi[[i]])
			attr(fixed.pred[[i]], which="names") <- NULL
		}

		#test
		if(is.null(tni)==FALSE){
			tfixed.pred <- list()
			for(i in 1:tn){
				tfixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.tDBi[[i]])
				attr(tfixed.pred[[i]], which="names") <- NULL
			}
		}
				
	#update the estimation of random effects parameters
		for(i in 1:n)
			bi[[r]][[i]] <- D[[r-1]]%*%t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]] %*% (wYi[[i]] - sqrt(wi[[i]])* fixed.pred[[i]])
		bi[r-1] <- list(NULL)		
		
	#########################################################
	####	STEP 2	#######################################         
	#########################################################

	#total predictions
		#learning
		for(i in 1:n)
			pred.Yi[[r]][[i]] <- fixed.pred[[i]] + Zi[[i]] %*% bi[[r]][[i]]
			pred.Yi[r-1] <- list(NULL)

		#test
		if(is.null(tni)==FALSE){
			for(i in 1:tn)
				tpred.Yi[[r]][[i]] <- tfixed.pred[[i]] + tZi[[i]] %*% bi[[r]][[i]]
				tpred.Yi[r-1] <- list(NULL)
		}

	#residuals at current iteration r
		for(i in 1:n)
			epsili[[r]][[i]] <- wYi[[i]] - sqrt(wi[[i]])*pred.Yi[[r]][[i]]
			epsili[r-1] <- list(NULL)
			
	#update level-1 variance component
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- crossprod(epsili[[r]][[i]]) + 
           			sigma.sq[r-1] * (ni[i] - sigma.sq[r-1]* sum(diag(inv.Vi[[r-1]][[i]])))
		sigma.sq[r] <- (1/N)*(sum(term))
		rm(term)
		gc(verbose=FALSE)

	#update level-2 variance component
		term <- list()
		term[[1]] <- tcrossprod(bi[[r]][[1]]) + 
                           (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[1]])%*% inv.Vi[[r-1]][[1]] %*% wZi[[1]] %*% D[[r-1]]
                            )
		for(i in 2:n) 
			term[[i]] <- term[[i-1]]+ tcrossprod(bi[[r]][[i]]) + 
                        (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]]%*% wZi[[i]]%*% D[[r-1]]
                        )
		term <- term[[n]]
		D[[r]] <- (1/n)*term
		rm(term)
		gc(verbose=FALSE)
		D[[r-1]] <- list(NULL) 
	
	#level-1 and level-2 variance components at current iteration r
		inv.Vi[[r]] <-list()
		for(i in 1:n){
			Vi[[i]] <- wZi[[i]] %*% D[[r]] %*% t(wZi[[i]])+sigma.sq[r]*diag(x = 1, nrow=ni[i], ncol = ni[i])
			inv.Vi[[r]][[i]] <- solve(Vi[[i]])
		}
		Vi <- list(NULL) 
		inv.Vi[[r-1]] <- list(NULL) 	
  
	#Update GLL value
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- t(epsili[[r]][[i]]) %*% solve(sigma.sq[r]*diag(x=1,nrow=ni[i],ncol=ni[i])) %*% epsili[[r]][[i]]
				+ t(bi[[r]][[i]]) %*% solve(D[[r]]) %*% bi[[r]][[i]]
				+ log(abs(D[[r]]))
				+ log(abs(sigma.sq[r]*diag(x=1, nrow=ni[i], ncol = ni[i])))
		GLL[r] <- sum(term)
		rm(term)
		gc(verbose=FALSE)

	#update the value of the change in GLL
		change[r] <- abs((GLL[r]-GLL[r-1]))


	}#end if and STOP repeating step 1 and 2
	else break
	
	#############################################################
	}#	END OF ITERATIONS BASED ON CONVERGENCE CRITERION#  
	#############################################################

	#################################
	#	Last L iterations	#
	#################################
		
		ri <- r
		while(r < (ri+L)){#repeat step 1 and 2

	#########################################################
	####	STEP 1	#######################################         
	#########################################################

	#update iteration number r
		r <- r+1
		if (verbose) 	
			message("micro-iter ", r)

	#update the length of the different lists
		length(pruned.tree)<-r
		length(pruned.tree.nb.leaves) <- r
		length(sigma.sq) <- r
		length(D) <- r
		length(inv.Vi) <- r
		length(pred.Yi)<- r	; pred.Yi[[r]]<-list()
		length(epsili) <- r	; epsili[[r]]<-list()
		length(bi) <- r
		length(GLL) <- r
		length(change) <- r
		if(is.null(tni)==FALSE){
			length(tpred.Yi) <- r	;tpred.Yi[[r]]<-list()
		}

	#update the transformed outcome, star.Yi
		for(i in 1:n){
			star.Yi[[i]] <- Yi[[i]] - Zi[[i]] %*% bi[[r-1]][[i]]	
		}	

	#use a standard weighted TREE model to predict the fixed part
	#############################################################
	#update starY values in mert.DB
		mert.DB$starY <- unlist(star.Yi)

	#large tree
		tree <- rpart(
			formula = mert.formula
			,data=mert.DB
			,method="anova"
			,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete 
			,minsplit=minsplit,minbucket=minbucket ,maxdepth=maxdepth
			,xval=xval
			,cp=cp
			,weight=w
		)
		#print(tree$cptable)
		min.index <- 1
		if(dim(tree$cptable)[1]>1) min.index <- which.min(tree$cptable[,"xerror"])

	#pruned tree
		pruned.tree[[r]] <- prune(tree, cp=tree$cptable[min.index,"CP"])
		rm(tree, min.index);gc(verbose=FALSE)

	#nb of terminal nodes of the pruned tree
		pruned.tree.nb.leaves[r] <- dim(table(pruned.tree[[r]]$where))

	#fixed part predictions in matrix format
		#learning	
		fixed.pred <- list()
		for(i in 1:n){
			fixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.DBi[[i]])
			attr(fixed.pred[[i]], which="names") <- NULL
		}

		#test
		if(is.null(tni)==FALSE){
			tfixed.pred <- list()
			for(i in 1:tn){
				tfixed.pred[[i]] <- predict(pruned.tree[[r]],newdata=mert.tDBi[[i]])
				attr(tfixed.pred[[i]], which="names") <- NULL
			}
		}
				
	#update the estimation of random effects parameters
		for(i in 1:n)
			bi[[r]][[i]] <- D[[r-1]]%*%t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]] %*% (wYi[[i]] - sqrt(wi[[i]])* fixed.pred[[i]])
		
	#######################################################
	####	STEP 2	#######################################         
	#######################################################

	#total predictions
		#learning
		for(i in 1:n)
			pred.Yi[[r]][[i]] <- fixed.pred[[i]] + Zi[[i]] %*% bi[[r]][[i]]

		#test
		if(is.null(tni)==FALSE){
			for(i in 1:tn)
				tpred.Yi[[r]][[i]] <- tfixed.pred[[i]] + tZi[[i]] %*% bi[[r]][[i]]
		}

	#residuals at current iteration r
		for(i in 1:n)
			epsili[[r]][[i]] <- wYi[[i]] - sqrt(wi[[i]])*pred.Yi[[r]][[i]]
			epsili[r-1] <- list(NULL)
			
	#update level-1 variance component
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- crossprod(epsili[[r]][[i]]) + 
           			sigma.sq[r-1] * (ni[i] - sigma.sq[r-1]* sum(diag(inv.Vi[[r-1]][[i]])))
		sigma.sq[r] <- (1/N)*(sum(term))
		rm(term)
		gc(verbose=FALSE)

	#update level-2 variance component
		term <- list()
		term[[1]] <- tcrossprod(bi[[r]][[1]]) + 
                           (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[1]])%*% inv.Vi[[r-1]][[1]] %*% wZi[[1]] %*% D[[r-1]]
                            )
		for(i in 2:n) 
			term[[i]] <- term[[i-1]]+ tcrossprod(bi[[r]][[i]]) + 
                        (	D[[r-1]] - 
                            	D[[r-1]] %*% t(wZi[[i]]) %*% inv.Vi[[r-1]][[i]]%*% wZi[[i]]%*% D[[r-1]]
                        )
		term <- term[[n]]
		D[[r]] <- (1/n)*term
		rm(term)
		gc(verbose=FALSE)
	
	#level-1 and level-2 variance components at current iteration r
		inv.Vi[[r]] <-list()
		for(i in 1:n){
			Vi[[i]] <- wZi[[i]] %*% D[[r]] %*% t(wZi[[i]])+sigma.sq[r]*diag(x = 1, nrow=ni[i], ncol = ni[i])
			inv.Vi[[r]][[i]] <- solve(Vi[[i]])
		}
		Vi <- list(NULL) 
		inv.Vi[[r-1]] <- list(NULL) 
  
	#Update GLL value
		term <- vector(mode="numeric",length=n)
		for(i in 1:n)
			term[i] <- t(epsili[[r]][[i]]) %*% solve(sigma.sq[r]*diag(x=1,nrow=ni[i],ncol=ni[i])) %*% epsili[[r]][[i]]
				+ t(bi[[r]][[i]]) %*% solve(D[[r]]) %*% bi[[r]][[i]]
				+ log(abs(D[[r]]))
				+ log(abs(sigma.sq[r]*diag(x=1, nrow=ni[i], ncol = ni[i])))
		GLL[r] <- sum(term)
		rm(term)
		gc(verbose=FALSE)

	#update the value of the change in GLL
		change[r] <- abs((GLL[r]-GLL[r-1]))

	#########################################
	}#	END OF LAST L ITERATIONS	#
	#########################################	



	#######################################################
	####	FINAL MODEL SELECTION	#########################        
	#######################################################

	#nb of terminal nodes at each last L iterations
		leaves <- numeric(length = L)
		for(i in 1:L)
			leaves[i] <- pruned.tree.nb.leaves[ri+i]

		#nb of the iteration corresponding to the final model 
		rf <-  ri + max(which(leaves==names(sort(-table(leaves)))[1]))

		if (verbose) 	
			message("micro-iter of selected MERT tree: ", rf)

	#########################################
	#	return only iter rf results	#
	#########################################


	#b1 and b2 
	b1 <- rep(NA,n)
	if(q==2)b2 <- rep(NA,n)	
	
	for(j in 1:n){
		if(q==1)b1[j] <- bi[[rf]][[j]]
		if(q==2){
			b1[j] <- bi[[rf]][[j]][1,1]
			b2[j] <- bi[[rf]][[j]][2,1]
		}
	}
	

	#D
	d11 <- D[[rf]][1,1]
	if(q==2){
		d22 <- D[[rf]][2,2]
		d12 <-D[[rf]][1,2]
	}
	
	#sigma.sq
	sigma.sq <- sigma.sq[rf]

	#pruned.tree (selected model)
	pruned.tree <- pruned.tree[[rf]]

	#nb of leaves of selected model
	pruned.tree.nb.leaves <- pruned.tree.nb.leaves[rf]

	#predictions based on selected model
	pred.Yi <- pred.Yi[[rf]]

	if(is.null(tni)==FALSE){
		tpred.Yi <- tpred.Yi[[rf]]
	}


	#output
	#######

	if(q==1){
		output <- list(pruned.tree,pruned.tree.nb.leaves ,pred.Yi,b1,d11,sigma.sq,rf)
		names(output) <- c("pruned.tree","pruned.tree.nb.leaves","pred.Yi","b1","d11","sigma.sq","rf")

		if(is.null(tni)==FALSE){
			output <- list(pruned.tree ,pruned.tree.nb.leaves ,pred.Yi,tpred.Yi,b1,d11,sigma.sq,rf)
			names(output) <- c("pruned.tree","pruned.tree.nb.leaves","pred.Yi","tpred.Yi","b1","d11","sigma.sq","rf")	
		}
	}

	if(q==2){
		output <- list(pruned.tree,pruned.tree.nb.leaves ,pred.Yi,b1,b2,d11,d22,d12,sigma.sq,rf)
		names(output) <- c("pruned.tree","pruned.tree.nb.leaves","pred.Yi","b1","b2","d11","d22","d12","sigma.sq","rf")

		if(is.null(tni)==FALSE){
			output <- list(pruned.tree ,pruned.tree.nb.leaves ,pred.Yi,tpred.Yi,b1,b2,d11,d22,d12,sigma.sq,rf)
			names(output) <- c("pruned.tree","pruned.tree.nb.leaves","pred.Yi","tpred.Yi","b1","b2","d11","d22","d12","sigma.sq","rf")	
		}
	}

	#clean memory
	#############
	rm(
		i,verbose
		,n,N,ni,p,q,Xi,Yi,Zi,star.Yi
		,wi
		,wYi,wZi
		,bi
		,bizero
		,fixed.pred ,pred.Yi,epsili
		,D ,Dzero ,sigma.sq ,sigmasqzero ,Vi,inv.Vi
		,F,L,niter,max.iter ,s.change,GLL,change,r ,ri ,rf
		,gr ,mert.DB ,mert.DBi 
		,pruned.tree ,pruned.tree.nb.leaves ,leaves 
		,maxcompete,maxdepth,maxsurrogate,minsplit,minbucket,surrogatestyle,usesurrogate,xval,cp
		,b1  ,d11  ,j ,mert.formula  ,xnam
	)

	if(q==2)rm(b2,d12 ,d22) ; gc(verbose=FALSE)

	if(is.null(tni)==FALSE){
		rm(
			tn,tN,tni,tXi,tYi,tZi ,mert.tDBi 
			,tfixed.pred
			,tpred.Yi
		)
		gc(verbose=FALSE)
	}

	rm(tn ,tN ,tni ,tXi ,tYi ,tZi) ;gc(verbose=FALSE)

	#return
	#######
	output

	#message("ls()",ls())

}#end of mert function

######################################################################################################
#################                End of mert Function		        ##############################
######################################################################################################




######################################################################################################
######################################################################################################
###########################		gmert Function		######################################
######################################################################################################
######################################################################################################

gmert <-function (
		 ni,Xi,Zi,Yi

		,tni=NULL
		,tXi=NULL
		,tZi=NULL
		,tYi=NULL

		,xnam

		,usesurrogate
		,surrogatestyle
		,maxsurrogate
		,maxcompete
		,minsplit
		,minbucket
		,maxdepth
		,xval
		,cp
		
		,F=1
		,s.change=1e-6
		,mert.niter=1
		,L=2
		,mert.verbose=T

		,sigmasqzero=sigmasqzero
		,Dzero=Dzero
		,bizero=bizero

		,family=binomial 
		,rc=1e-6
		,gmert.niter = 1
		,S=2	
		,verbose
	) 

{

	#################
	# Macro step 0	#
	#################
		M <- 1
					
		if (verbose) 
			#print MACRO STEP ZERO
			message("### MACRO ITERATION ZERO")

		#family gives the error distribution and link function to be used in the model. 
		if (is.character(family)) 
			family <- get(family)
		if (is.function(family)) 
			family <- family()
		if (is.null(family$family)) {
			print(family)
			stop("'family' not recognized")
		}
		fam <- family

		#data preparation
		N<-sum(ni)
		n<-length(ni)
		tn<-length(tni)
		tN<-sum(tni)

		q <- dim(Zi[[1]])[2]
				
		#data as data.frame
		DBi <- list(); length(DBi) <- n
		for(i in 1:n)
			DBi[[i]] <- as.data.frame(cbind(Yi[[i]], Xi[[i]]))

		for(i in 2:n){
			DBi[[i]] <- as.data.frame(rbind(DBi[[i-1]], DBi[[i]]))
			DBi[i-1] <- list(NULL)
		}
		DB <- as.data.frame(DBi[[n]])


		rm(DBi) ;gc(verbose=FALSE)

		#fit micro or pseudo model at each macro iter
		fit<-list();length(fit)<-M
		fit[M] <- list(NULL)

			#initial estimated means (probabilities)
			mu<-list();length(mu)<-M
			mu[[M]] <- numeric(length=N)
			for(i in 1:N){
				if(DB[i,1] == 1) mu[[M]][i] <- 0.75
				if(DB[i,1] == 0) mu[[M]][i] <- 0.25
			}

			#initial linear predictor
			eta<-list();length(eta)<-M
			eta[[M]] <- fam$linkfun(mu[[M]]) 

			#initial estimated weights (=1/var(pseudo response) = mu*(1-mu))
				#as one vector of length N
				w <-list();length(w)<-M			
				w[[M]]   <- (fam$mu.eta(eta[[M]]))^2/fam$variance(mu[[M]])			
					
				#as a list of n vectors of length ni			
				wi<-list(); length(wi)<-n
				for(i in 1:n) wi[[i]] <- numeric(length=ni[i])
				wi[[1]] <- w[[M]][1:ni[1]]
				b <- 0
				for(i in 2:n){
			 		b <- b+ni[i-1]
			 		wi[[i]] <- w[[M]][ (b+1) : (b + ni[i]) ] 
				}
				rm(b)
				gc(verbose=FALSE)


			#initial linearized dependent variable or pseudo response
				#as one vector of length N 
				yl<-list();length(yl)<-M
				yl[[M]] <- eta[[M]] + (DB[,1] - mu[[M]]) / fam$mu.eta(eta[[M]])			
					

				#as a list of n vectors of length ni
				yli<-list();length(yli)<-n
				for(i in 1:n) yli[[i]]<- matrix(0,nrow=ni[i],ncol=1)
				yli[[1]] <- as.matrix(yl[[M]][1:ni[1]])
				b <-0
				for(i in 2:n){
			 		b <- b+ni[i-1]
			 		yli[[i]] <- as.matrix( yl[[M]][ (b+1) : (b + ni[i]) ] )
				}
				rm(b)
				gc(verbose=FALSE)

			#convergence 
			cv.cr <- numeric();length(cv.cr) <- M 	
			cv.cr[1] <- 0

	#########################
	# MACRO iterations loop #
	#########################
	for (l in 1:gmert.niter) {

		#macro iter 
		M <- M+1 
	
		#print macro iter number
		if (verbose) 
			message("### MACRO ITERATION ", M-1)

		#update object lengths
			length(fit) <- M
			length(eta) <- M
			length(mu) <- M
			length(w)<- M
			length(yl) <- M
			length(cv.cr) <- M	

		#fit mert pseudo or micro model
			fit[[M]] <- mert(
					 ni=ni,Xi=Xi,Zi=Zi 

					,Yi=yli			#linearized Y as a dependent variable

					,tni=tni,tXi=tXi,tZi=tZi 
					,tYi=tyi

					,xnam

					,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete
					,minsplit=minsplit,minbucket=minbucket
					,maxdepth=maxdepth
					,xval=xval
					,cp=cp

					,F=F
					,s.change
					,niter=mert.niter
					,L=L
					,verbose

					,sigmasqzero
					,Dzero
					,bizero

					,wi=wi					
			)

		#update eta 										
			eta[[M]] <- unlist(fit[[M]]$pred.Yi)	 
			
		#update mu
			mu[[M]] <- fam$linkinv(eta[[M]])		

		#update weights w and wi
			w[[M]]  <- (fam$mu.eta(eta[[M]]))^2/fam$variance(mu[[M]])		
			w[M-1] <-list(NULL)

			wi[[1]] <- w[[M]][1:ni[1]]
			b <- 0
			for(i in 2:n){
			 	b <- b+ni[i-1]
			 	wi[[i]] <- w[[M]][ (b+1) : (b + ni[i]) ] 
			}
			rm(b)
			gc(verbose=FALSE)
	
		#update linearized dependent variable yl and yli
			yl[[M]] <- eta[[M]] + (DB[,1] - mu[[M]]) / fam$mu.eta(eta[[M]])		
			
			yli[[1]] <- as.matrix(yl[[M]][1:ni[1]])
			b <- 0
			for(i in 2:n){
			 	b <- b+ni[i-1]
			 	yli[[i]] <- as.matrix( yl[[M]][ (b+1) : (b + ni[i]) ] )
			}
			rm(b)
			gc(verbose=FALSE)

		#update initial values of bi, D, and sigma.sq
		#############################################
			sigmasqzero <- fit[[M]]$sigma.sq	

			if(q==1)Dzero <- diag(fit[[M]]$d11, ncol=1, nrow=1)
			if(q==2)Dzero <- matrix(c(fit[[M]]$d11,fit[[M]]$d12,fit[[M]]$d12,fit[[M]]$d22) ,ncol=2, nrow=2)

			if(q==1){
				for(i in 1:n) 
					bizero[[i]]<- as.matrix(fit[[M]]$b1[i], ncol=1, nrow=1)
			}

			if(q==2){
				for(i in 1:n) 
					bizero[[i]] <- as.matrix(c(fit[[M]]$b1[i], fit[[M]]$b2[i]), ncol=1, nrow=2)
			}

		#convergence criterion
		######################
			cv.cr[M]<- (sum((eta[[M]] - eta[[M-1]])^2)) / (sum((eta[[M]])^2))

	
			if (sum((eta[[M]] - eta[[M-1]])^2) < rc * sum((eta[[M]])^2)) {		
	 		 if (verbose) message("### MACRO-CONVERGENCE AT MACRO-ITER ", M-1, " ###")
	 		 break
			}
		
	##############################	
	}# end macro iterations loop # 
	##############################	


	################################
	# Last S additional macro iter #
	################################
	Mi <- M
	while(M < (Mi+S)){#repeat Macro iter loop

		#########################
		# MACRO iterations loop #
		#########################

		#macro iter number + 1
		M<-M+1 
	
	
		#print macro iter number
		if (verbose) 
			message("### MACRO ITERATION ", M-1)

		#update object lengths
			length(fit) <- M
			length(eta) <- M
			length(mu) <- M
			length(w)<- M
			length(yl) <- M
			length(cv.cr) <- M
			
		#fit mert pseudo or micro model

			message("ho passato il primo mert")
			fit[[M]] <- mert(
					 ni=ni,Xi=Xi,Zi=Zi 

					,Yi=yli			

					,tni=tni,tXi=tXi,tZi=tZi 
					,tYi=tyi

					,xnam

					,usesurrogate=usesurrogate,surrogatestyle=surrogatestyle,maxsurrogate=maxsurrogate,maxcompete=maxcompete
					,minsplit=minsplit,minbucket=minbucket
					,maxdepth=maxdepth
					,xval=xval
					,cp=cp

					,F=F
					,s.change
					,niter=mert.niter
					,L=L
					,verbose 

					,sigmasqzero
					,Dzero
					,bizero

					,wi=wi					
			)

		#update eta 										
			eta[[M]] <- unlist(fit[[M]]$pred.Yi)	 
								
		#update mu
			mu[[M]] <- fam$linkinv(eta[[M]])		

		#update weights w and wi
			w[[M]]  <- (fam$mu.eta(eta[[M]]))^2/fam$variance(mu[[M]])		
 
			wi[[1]] <- w[[M]][1:ni[1]]
			b <- 0
			for(i in 2:n){
			 	b <- b+ni[i-1]
			 	wi[[i]] <- w[[M]][ (b+1) : (b + ni[i]) ] 
			}
			rm(b)
			gc(verbose=FALSE)
			
		#update linearized dependent variable yl and yli
			yl[[M]] <- eta[[M]] + (DB[,1] - mu[[M]]) / fam$mu.eta(eta[[M]])		
				 
									
			yli[[1]] <- as.matrix(yl[[M]][1:ni[1]])
			b <- 0
			for(i in 2:n){
			 	b <- b+ni[i-1]
			 	yli[[i]] <- as.matrix( yl[[M]][ (b+1) : (b + ni[i]) ] )
			}
			rm(b)
			gc(verbose=FALSE)

		#update initial values of bi, D, and sigmasq
		############################################
			sigmasqzero <- fit[[M]]$sigma.sq	

			if(q==1)Dzero <- diag(fit[[M]]$d11, ncol=1, nrow=1)
			if(q==2)Dzero <- matrix(c(fit[[M]]$d11,fit[[M]]$d12,fit[[M]]$d12,fit[[M]]$d22) ,ncol=2, nrow=2)

			if(q==1){
				for(i in 1:n) 
					bizero[[i]]<- as.matrix(fit[[M]]$b1[i], ncol=1, nrow=1)
			}

			if(q==2){
				for(i in 1:n) 
					bizero[[i]] <- as.matrix(c(fit[[M]]$b1[i], fit[[M]]$b2[i]), ncol=1, nrow=2)
			}


		#convergence criterion
			cv.cr[M]<- (sum((eta[[M]] - eta[[M-1]])^2)) / (sum((eta[[M]])^2))	
	
	#########################################
	}#	END OF LAST S Macro ITERATIONS	#
	#########################################


	##############################
	#Final GMERT model selection #
	##############################
	#Final GMERT model = last model with number of leaves equal to mode value of leaves number among the last S additional macro iter
	
	#nb of terminal nodes at each macro-iter of the last S
		gmert.leaves <- numeric(length = S)
		for(i in 1:S)
			gmert.leaves[i] <- fit[[Mi+i]]$pruned.tree.nb.leaves

		#Macro-iteration corresponding to the selected GMERT model ("- 1" on order to exclude macro-iter 0) !!!
			gmert.rf <-  Mi + max(which(gmert.leaves==names(sort(-table(gmert.leaves)))[1])) - 1 	
			
		if (verbose)
			message("Macro-iter of selected GMERT model based on modal nb of leaves at last s macro-iter : ", gmert.rf)


	################
	# gmert output #
	################
	#gmert.out: a list of length M-1 giving results of each macro-iter (macro-iter 0 is excluded) !!!
	#gmert.out[[gmert.rf]] gives results of selected GMERT Model

	gmert.out <- list() ; length(gmert.out) <- M - 1

	for(i in 2:M){

		#cv.cr
		cvcr <- cv.cr[i]

		#rf.mert
		rf.mert <- fit[[i]]$rf
 
		#gmert tree
		gmert.tree <- fit[[i]]$pruned.tree	

		#gmert nb of leaves
		gmert.nb.leaves <- fit[[i]]$pruned.tree.nb.leaves	
		

		#gmert estimated level-1 variance parameter sigma.sq
		gmert.sigma.sq <- fit[[i]]$sigma.sq			

		#gmert estimated cluster-level var-cov parameters
		gmert.d11 <- fit[[i]]$d11	
		if(q==2){
			gmert.d22 <- fit[[i]]$d22	
			gmert.d12 <- fit[[i]]$d12	
		}

		#gmert conditional prob (muij)
		gmert.l.c.prob <- mu[[i]]	
		if(is.null(tni)==FALSE)gmert.t.c.prob <- fam$linkinv( unlist(fit[[i]]$tpred.Yi) )        
			

		#gmert random effects b1 and b2
		gmert.b1 <- fit[[i]]$b1
		if(is.null(tni)==FALSE)gmert.b2 <- fit[[i]]$b2


		if(q==1){
			output <- list(cvcr ,rf.mert ,gmert.tree ,gmert.nb.leaves ,gmert.b1 ,gmert.sigma.sq ,gmert.d11 ,gmert.l.c.prob ,gmert.rf)
			names(output) <- c("cvcr" ,"rf.mert" ,"gmert.tree" ,"gmert.nb.leaves" ,"gmert.b1","gmert.sigma.sq" ,"gmert.d11" ,"gmert.l.c.prob" ,"gmert.rf")

			if(is.null(tni)==FALSE){
				output <- list(cvcr ,rf.mert ,gmert.tree ,gmert.nb.leaves ,gmert.b1 ,gmert.sigma.sq ,gmert.d11 ,gmert.l.c.prob ,gmert.rf ,gmert.t.c.prob)
				names(output) <- c("cvcr" ,"rf.mert" ,"gmert.tree" ,"gmert.nb.leaves" ,"gmert.b1" ,"gmert.sigma.sq" ,"gmert.d11" ,"gmert.l.c.prob" ,"gmert.rf" ,"gmert.t.c.prob")	
			}
		}


		if(q==2){
			output <- list(cvcr ,rf.mert ,gmert.tree ,gmert.nb.leaves ,gmert.b1,gmert.b2 ,gmert.sigma.sq ,gmert.d11 ,gmert.l.c.prob ,gmert.rf ,gmert.d22 ,gmert.d12)
			names(output) <- c("cvcr" ,"rf.mert" ,"gmert.tree" ,"gmert.nb.leaves" ,"gmert.b1" ,"gmert.b2" ,"gmert.sigma.sq" ,"gmert.d11" ,"gmert.l.c.prob" ,"gmert.rf" ,"d22gmert." ,"d12gmert.")

			if(is.null(tni)==FALSE){
				output <- list(cvcr ,rf.mert ,gmert.tree ,gmert.nb.leaves ,gmert.b1,gmert.b2,gmert.sigma.sq ,gmert.d11 ,gmert.l.c.prob ,gmert.rf ,gmert.t.c.prob ,gmert.d22 ,gmert.d12)
				names(output) <- c("cvcr" ,"rf.mert" ,"gmert.tree" ,"gmert.nb.leaves" ,"gmert.b1" ,"gmert.b2","gmert.sigma.sq" ,"gmert.d11" ,"gmert.l.c.prob" ,"gmert.rf" ,"gmert.t.c.prob" ,"gmert.d22" ,"gmert.d12")	
			}
		}


		gmert.out[[i-1]] <- output 	
	

	}#end gmert.out
	###############

	#clean memory
	#############
	rm( # c'era q qui ma io l'ho tolto (Chiara)
		bizero,cp,cv.cr,DB,Dzero,eta,F,fam,family,fit,i,l,L,M ,maxcompete,maxdepth,maxsurrogate
		,minsplit,minbucket,mu,mert.niter,mert.verbose,gmert.niter,n,N,ni,sigmasqzero,surrogatestyle,usesurrogate
		,verbose,w,wi,Xi,xval,Yi,yl,yli,Zi
		,xnam
		,gmert.leaves ,Mi ,S ,gmert.rf
		,gmert.tree,gmert.nb.leaves ,gmert.sigma.sq ,gmert.d11 ,gmert.l.c.prob
		,gmert.b1 
		,output
		,cvcr
		,rf.mert
		,rc
		,s.change
	)

	if(is.null(tni)==FALSE){
		rm(
			tn,tN,tni,tXi,tYi,tZi
			,gmert.t.c.prob
		)
	}

	rm(tn ,tN ,tni ,tXi ,tYi ,tZi) ; gc(verbose=FALSE)

	if(q==2)rm(gmert.d22 ,gmert.d12 ,gmert.b2)

	gc(verbose=FALSE)

	#return
	#######
	gmert.out 

	#message("ls()",ls())

}#end of gmert function

######################################################################################################
#################                End of gmert Function		        ##############################
######################################################################################################					