# prove di simulazione (dataset simulati)
# v. GMERT tesi
# Versione Compatta

rm(list=ls())

setwd("~/LUCA/Articolo Luca/R")

require(rpart)
require(rpart.plot)
require(REEMtree)
require(nlme)
require(lme4)
require(boot)
require(MASS)
require(reshape)
require(plyr)

source("genera.dati.R")
source("find.cutoff.R")
source("mymixedtree.R")

maxiter = 50

PMAD_tree = matrix(rep(0,maxiter*10), nrow=maxiter, ncol=10)
PMAD_myI <- PMCR_myI  <- PMCR_myS <- PMAD_myS <- PMCR_tree <- PMAD_glm <- PMCR_glm <- PMAD_tree

pb <- txtProgressBar(min = 0, max = maxiter*10, style = 3) #text based bar


Predictors <- "x1+x2+x3+x4+x5+x6+x7+x8"

random = rep(0,10)
random[1:6] =  '~  (1|cluster)'
random[7:10] =  '~  (1+z1|cluster)'
RI <- ~  (1|cluster)
RIS <- ~  (1+z1|cluster)

###########################
##### SIMULAZIONE N.2  ####
###########################
set.seed(1993)

for(ITER in 1:maxiter) {
  
  n.train = 10
  n.test = 50
  K = 50
  N.test = K*n.test
  
  sim = genera.dati(n.train,n.test,K)
  dati = sim$dati
  
  
  for(DGP in 1:10){
    
    Target = paste('y',DGP, sep='')
    F.tree = formula(paste(c(Target, Predictors), collapse = "~"))
    mu = paste('mu',DGP, sep='')
    mutrue = subset(dati[,mu], !dati$is.train)
    ytrue = subset(dati[,Target], !dati$is.train)
    mean.class = mean(subset(dati[,Target], dati$is.train))
    
    ## REGRESSION TREE
    tree = rpart(F.tree, data = dati, subset = is.train,
                  method = "class" )
    mupred = as.data.frame(predict(tree, newdata = subset(dati, !dati$is.train)))
    mupred = mupred$`1`
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_tree[ITER,DGP] = pmad*100
    ypred = predict(tree, newdata = subset(dati, !dati$is.train), type = "class")
    ypred = as.integer(ypred) -1
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_tree[ITER,DGP] = pmcr*100
    
    ## GLMM with known structure
    fixed = paste(c(Target, "as.factor(leaf)"), collapse = "~")
    rand = formula(random[DGP])
    F.glmm = formula(paste(c(fixed, rand[[2]]), collapse = "+"))
    glm = glmer(F.glmm, data = dati, subset = is.train,
                 family = "binomial" )
    mupred = as.data.frame(predict(glm, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glm[ITER,DGP] = pmad*100
    pred.train = predict(glm, type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glm[ITER,DGP] = pmcr*100
    
    ## Mixed Tree, Random Intercept
    my = mymixedtree(F.tree, dataset = dati, random = RI, 
                     subset = "is.train", family = binomial)
    mupred = as.data.frame(predict(my, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myI[ITER,DGP] = pmad*100
    pred.train = predict(my, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myI[ITER,DGP] = pmcr*100
    
    ## Mixed Tree, Random Intercept&Slope
    myS = mymixedtree(F.tree, dataset = dati, random = RIS, 
                      subset = "is.train", family = binomial)
    mupred = as.data.frame(predict(myS, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myS[ITER,DGP] = pmad*100
    pred.train = predict(myS, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myS[ITER,DGP] = pmcr*100
    
    Sys.sleep(0.0001) 
    setTxtProgressBar(pb, (ITER-1)*10+DGP)
  }
  
}
close(pb)
print("******PMAD******")
round(colMeans(PMAD_myS), 3)
round(colMeans(PMAD_myI), 3)
round(colMeans(PMAD_tree), 3)
round(colMeans(PMAD_glm), 3)

print("******PMCR******")
round(colMeans(PMCR_myS), 3)
round(colMeans(PMCR_myI), 3)
round(colMeans(PMCR_tree), 3)
round(colMeans(PMCR_glm), 3)


#################
# load("~/LUCA/LUCA_TESI/3. Mixed Trees/GREEM simulazioni/SIMULAZIONE_50cluster50run.RData")
require(plyr)
require(reshape)

PMAD_myS <- data.frame(PMAD_myS, run = 1:10, method = "RIS")
colnames(PMAD_myS) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                    "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                    "method")
df1 = melt(PMAD_myS, id.vars = c("run","method"), variable_name = "DGP")

PMAD_myI <- data.frame(PMAD_myI, run = 1:10, method = "RI")
colnames(PMAD_myI) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
df2 = melt(PMAD_myI, id.vars = c("run","method"), variable_name = "DGP")

PMAD_tree <- data.frame(PMAD_tree, run = 1:10, method = "Std")
colnames(PMAD_tree) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
df3 = melt(PMAD_tree, id.vars = c("run","method"), variable_name = "DGP")

PMAD_glm <- data.frame(PMAD_glm, run = 1:10, method = "MElog")
colnames(PMAD_glm) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
df4 = melt(PMAD_glm, id.vars = c("run","method"), variable_name = "DGP")


df = rbind(df3,df2,df1,df4)

summPMAD = ddply(df, .(DGP, method), function(x)   data.frame(mean = round(mean(x$value),2), 
                                                            sd = round(sd(x$value),2),
                                                         min =round(min(x$value),2),
                                                         max =round(max(x$value),2) ))
summPMAD



PMCR_myS <- data.frame(PMCR_myS, run = 1:10, method = "RIS")
colnames(PMCR_myS) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
dff1 = melt(PMCR_myS, id.vars = c("run","method"), variable_name = "DGP")

PMCR_myI <- data.frame(PMCR_myI, run = 1:10, method = "RI")
colnames(PMCR_myI) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
dff2 = melt(PMCR_myI, id.vars = c("run","method"), variable_name = "DGP")

PMCR_tree <- data.frame(PMCR_tree, run = 1:10, method = "Std")
colnames(PMCR_tree) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                         "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                         "method")
dff3 = melt(PMCR_tree, id.vars = c("run","method"), variable_name = "DGP")

PMCR_glm <- data.frame(PMCR_glm, run = 1:10, method = "MElog")
colnames(PMCR_glm) <- c("DGP1","DGP2","DGP3","DGP4","DGP5",
                        "DGP6","DGP7","DGP8","DGP9","DGP10", "run", 
                        "method")
dff4 = melt(PMCR_glm, id.vars = c("run","method"), variable_name = "DGP")


dff = rbind(dff3,dff2,dff1,dff4)

summPMCR = ddply(dff, .(DGP, method), function(x)   data.frame(mean = round(mean(x$value),2), 
                                                          sd = round(sd(x$value),2),
                                                          min =round(min(x$value),2),
                                                          max =round(max(x$value),2) ))
summPMCR

colnames(summPMCR) <- colnames(summPMAD) <- c("DGP","Fitted model", "mean",   "sd" ,    "min" ,   "max" )

summ = cbind(summPMAD[,2:6], summPMCR[,3:6])

library(xtable)
options(xtable.floating = F)
options(xtable.timestamp = "")

tab = xtable(summ)
print(tab, include.rownames = F)
