# GOAL: implementclassification strategy and find cutpoint value 
# used to classify the observations in the test set

# Inputs:
#   pred.train contiene la lista dei "fitted values" per il training test
#   mean.class la proporzione nel training dataset di osservazioni della classe "1"

find.cutoff <- function(fitted.val = fitted.val, mean.class = mean.class){
  
  # 1) Sort the distinct predicted probabilities of the observations in the training set
  train.size <- length(fitted.val)
  cuts <- sort(unique(fitted.val))
  cuts.size = length(cuts)
  
  # 2) Classify the observations in the training set using in turn each one of these distinct
  # predicted probabilities as a cutpoint ; classify as class 1 each observation in the training
  # set that has a predicted probability equal to or higher than the cutpoint value,
  mat = data.frame(matrix(rep(0, train.size*cuts.size), nrow=train.size, ncol = cuts.size))
  for(i in 1:train.size){
    for(j in 1:cuts.size){
      if(fitted.val[i] >= cuts[j])
        mat[i,j] = 1
    }
  }
  # ottengo una matrice che ha in ogni riga come classifico l'osservazione a seconda del cutpoint
  
  # 3) Compute the proportions of class 1 that result from each one of the above cutpoint
  # values,
  prop = colMeans(mat)
  
  # 4) Find the predicted probability among those in step 1 that yields the closest 
  # proportion of class 1 to the actual proportion of class 1 in the training set,
  min = which.min(abs(prop - mean.class))
  
  # 5) Use this predicted probability as the cutpoint value in order to classify 
  # the observations in the test set,
  cuts[min]
  
}
