genera.dati.pois.tree <- function(n.train, n.test, K){
  # INPUT:
  # n.train = n of training observations in each cluster
  # n.test = n of test observations in each cluster
  # K = n of clusters
  # OUTPUT:
  # dati = dataframe that for every observation contains fixed- and random-effects covariates
  #           conditional probability of success for the 10 different DGPs
  #           output binary response for the 10 different DGPs
  # B = 
  require(boot)
  require(MASS)
  
  n = sum(n.train + n.test)
  n.k = n.train + n.test
  
  # group index:
  is.train=as.matrix(c(rep(TRUE,n.train[1]), rep(FALSE, n.test[1])))
  cluster = as.matrix(rep(1,n.k[1]))
  
  for(i in 2:K){
    is.train= rbind(is.train,(as.matrix(c(rep(TRUE,n.train[i]), rep(FALSE, n.test[i])))))
    cluster = rbind(cluster, as.matrix(rep(i,n.k[i])))
  }
  
  cluster = factor(cluster)
  # conditional prob. of success:
  #phi1 = c(0.1,0.2,0.8,0.2,0.8,0.9)
  #phi2 = c(0.2,0.4,0.7,0.3,0.6,0.8)
  
  # conditional lambda:
  phi1 = c(2,4,6,4,2,8)
  phi2 = c(4,6,8,6,4,10)
  
  # fixed-effects predictors:
  x1 = runif(n,0,5)
  x2 = runif(n,0,5)
  x3 = runif(n,0,5)
  x4 = runif(n,0,5)
  x5 = runif(n,0,5)
  x6 = runif(n,0,5)
  x7 = runif(n,0,5)
  x8 = runif(n,0,5)
  
  # x1=rnorm(n,mean=0, sd=1)
  # x2=rnorm(n,mean=0, sd=1)
  # x3=rnorm(n,mean=0, sd=1)
  # x4=rnorm(n,mean=0, sd=1)
  # x5=rnorm(n,mean=0, sd=1)
  # x6=rnorm(n,mean=0, sd=1)
  # x7=rnorm(n,mean=0, sd=1)
  # x8=rnorm(n,mean=0, sd=1)
  
  
  # random-effects predictors:
  z0 = rep(1,n)
  z1 = x1
  # variance-covariance matrix of the random effects:
  d3 = 2 #4
  d4 = 5 #10
  d5 = 0.25 #0.5
  d6 = 2 #4
  d7 = matrix(c(2,0,0,0.05), nrow=2, ncol=2, byrow = T)
  d8 = matrix(c(5,0,0,0.25), nrow=2, ncol=2, byrow = T)
  d9 = matrix(c(0.25,0,0,0.01), nrow=2, ncol=2, byrow = T)
  d10 = matrix(c(2,0,0,0.05), nrow=2, ncol=2, byrow = T)
  # generate random-effect coefficients:
  b1 = b2 = 0
  b3 = rnorm(n = K, 0, sqrt(d3))
  b4 = rnorm(n = K, 0, sqrt(d4))
  b5 = rnorm(n = K, 0, sqrt(d5))
  b6 = rnorm(n = K, 0, sqrt(d6))
  b7 = mvrnorm(n = K, mu = c(0,0), Sigma = d7)
  b8 = mvrnorm(n = K, mu = c(0,0), Sigma = d8)  
  b9 = mvrnorm(n = K, mu = c(0,0), Sigma = d9)
  b10 = mvrnorm(n = K, mu = c(0,0), Sigma = d10)
  
  #identify leaf:
  leaf = rep(0,n)
  for(i in 1:n){
    if(x1[i]<=5 & x2[i] <=5)
      leaf[i] = 1
    if(x1[i]<=5 & x2[i] > 5 & x4[i] <= 5)
      leaf[i] = 2
    if(x1[i]<=5 & x2[i] > 5 & x4[i] > 5)
      leaf[i] = 3
    if(x1[i]>5 & x3[i] <= 5 & x5[i] <= 5)
      leaf[i] = 4
    if(x1[i]>5 & x3[i] <= 5 & x5[i] > 5)
      leaf[i] = 5
    if(x1[i]>5 & x3[i] > 5)
      leaf[i] = 6
  }
  
  dati = data.frame(
    is.train,
    cluster,
    x1,x2,x3,x4,x5,x6,x7,x8,z0,z1,leaf
  )
  
  for(i in 1:n){
    k = dati$cluster[i]
    l = dati$leaf[i]
    
    # compute conditional probability mu_ij
    dati$mu1[i] = exp(log(phi1[l]) + z0[i]*b1)
    dati$mu2[i] = exp(log(phi2[l]) + z0[i]*b2)
    dati$mu3[i] = exp(log(phi1[l]) + z0[i]*b3[k])
    dati$mu4[i] = exp(log(phi1[l]) + z0[i]*b4[k])
    dati$mu5[i] = exp(log(phi2[l]) + z0[i]*b5[k])
    dati$mu6[i] = exp(log(phi2[l]) + z0[i]*b6[k])
    dati$mu7[i] = exp(log(phi1[l]) + z0[i]*b7[k,1] + z1[i]*b7[k,2])
    dati$mu8[i] = exp(log(phi1[l]) + z0[i]*b8[k,1] + z1[i]*b8[k,2])
    dati$mu9[i] = exp(log(phi2[l]) + z0[i]*b9[k,1] + z1[i]*b9[k,2])
    dati$mu10[i] = exp(log(phi2[l]) + z0[i]*b10[k,1] + z1[i]*b10[k,2])
    # simulate from a poisson with lambda = mu_ij
    dati$y1[i] = rpois(lambda=dati$mu1[i],  n=1)
    dati$y2[i] = rpois(lambda=dati$mu2[i],  n=1)
    dati$y3[i] = rpois(lambda=dati$mu3[i],  n=1)
    dati$y4[i] = rpois(lambda=dati$mu4[i],  n=1)
    dati$y5[i] = rpois(lambda=dati$mu5[i],  n=1)
    dati$y6[i] = rpois(lambda=dati$mu6[i],  n=1)
    dati$y7[i] = rpois(lambda=dati$mu7[i],  n=1)
    dati$y8[i] = rpois(lambda=dati$mu8[i],  n=1)
    dati$y9[i] = rpois(lambda=dati$mu9[i],  n=1)
    dati$y10[i] = rpois(lambda=dati$mu10[i], n=1)
  }
  
  # par(mfrow=c(5,2))
  # hist(dati$y1)
  # hist(dati$y2)
  # hist(dati$y3)
  # hist(dati$y4)
  # hist(dati$y5)
  # hist(dati$y6)
  # hist(dati$y7)
  # hist(dati$y8)
  # hist(dati$y9)
  # hist(dati$y10)
  
  
  
  B = data.frame(cluster=1:K, b1,b2,b3,b4,b5,b6,b7=b7,b8=b8,b9=b9,b10 =b10)
  
  result <- list(dati = dati, B = B)
  
  return(result)
}




genera.dati.pois <- function(n.train, n.test, K){
  # INPUT:
  # n.train = n of training observations in each cluster
  # n.test = n of test observations in each cluster
  # K = n of clusters
  # OUTPUT:
  # dati = dataframe that for every observation contains fixed- and random-effects covariates
  #           conditional probability of success for the 10 different DGPs
  #           output binary response for the 10 different DGPs
  # B = 
  require(boot)
  require(MASS)
  
  n = sum(n.train + n.test)
  n.k = n.train + n.test
  
  # group index:
  is.train=as.matrix(c(rep(TRUE,n.train[1]), rep(FALSE, n.test[1])))
  cluster = as.matrix(rep(1,n.k[1]))
  
  for(i in 2:K){
    is.train= rbind(is.train,(as.matrix(c(rep(TRUE,n.train[i]), rep(FALSE, n.test[i])))))
    cluster = rbind(cluster, as.matrix(rep(i,n.k[i])))
  }
  
  cluster = factor(cluster)
  # conditional prob. of success:
  #phi1 = c(0.1,0.2,0.8,0.2,0.8,0.9)
  #phi2 = c(0.2,0.4,0.7,0.3,0.6,0.8)
  
  # fixed-effects predictors:
  x1 = runif(n,1,4)
  x2 = runif(n,1,4)
  x3 = runif(n,1,4)
  x4 = runif(n,0,5)
  x5 = runif(n,0,5)
  x6 = runif(n,0,5)
  x7 = runif(n,0,5)
  x8 = runif(n,0,5)
  
  # x2=rnorm(n,mean=0, sd=1)
  # x3=rnorm(n,mean=0, sd=1)
  # x4=rnorm(n,mean=0, sd=1)
  # x5=rnorm(n,mean=0, sd=1)
  # x6=rnorm(n,mean=0, sd=1)
  # x7=rnorm(n,mean=0, sd=1)
  # x8=rnorm(n,mean=0, sd=1)

  
  # random-effects predictors:
  z0 = rep(1,n)
  z1 = x1
  # variance-covariance matrix of the random effects:
  d3 = 2 #4
  d4 = 5 #10
  d5 = 0.25 #0.5
  d6 = 2 #4
  d7 = matrix(c(2,0,0,0.05), nrow=2, ncol=2, byrow = T)
  d8 = matrix(c(5,0,0,0.25), nrow=2, ncol=2, byrow = T)
  d9 = matrix(c(0.25,0,0,0.01), nrow=2, ncol=2, byrow = T)
  d10 = matrix(c(2,0,0,0.05), nrow=2, ncol=2, byrow = T)
  # generate random-effect coefficients:
  b1 = b2 = 0
  b3 = rnorm(n = K, 0, sqrt(d3))
  b4 = rnorm(n = K, 0, sqrt(d4))
  b5 = rnorm(n = K, 0, sqrt(d5))
  b6 = rnorm(n = K, 0, sqrt(d6))
  b7 = mvrnorm(n = K, mu = c(0,0), Sigma = d7)
  b8 = mvrnorm(n = K, mu = c(0,0), Sigma = d8)  
  b9 = mvrnorm(n = K, mu = c(0,0), Sigma = d9)
  b10 = mvrnorm(n = K, mu = c(0,0), Sigma = d10)
  
  # identify leaf:
  # leaf = rep(0,n)
  # for(i in 1:n){
  #   if(x1[i]<=5 & x2[i] <=5)
  #     leaf[i] = 1
  #   if(x1[i]<=5 & x2[i] > 5 & x4[i] <= 5)
  #     leaf[i] = 2
  #   if(x1[i]<=5 & x2[i] > 5 & x4[i] > 5)
  #     leaf[i] = 3
  #   if(x1[i]>5 & x3[i] <= 5 & x5[i] <= 5)
  #     leaf[i] = 4
  #   if(x1[i]>5 & x3[i] <= 5 & x5[i] > 5)
  #     leaf[i] = 5
  #   if(x1[i]>5 & x3[i] > 5)
  #     leaf[i] = 6
  # }
  
  dati = data.frame(
    is.train,
    cluster,
    x1,x2,x3,x4,x5,x6,x7,x8,z0,z1
  )
  
  for(i in 1:n){
    k = dati$cluster[i]
    #l = dati$leaf[i]
    # small fixed -0.6*x1[i] -0.15*x2[i] - 0.1*x3[i]
    # large fixed -1.2*x1[i] -0.3*x2[i] - 0.2*x3[i]
    # compute conditional probability mu_ij
    dati$mu1[i] = exp((0.6*x1[i] + 0.3*x2[i] + 0.2*x3[i]) + z0[i]*b1)
    dati$mu2[i] = exp((0.3*x1[i] +0.15*x2[i] + 0.1*x3[i]) + z0[i]*b2)
    dati$mu3[i] = exp((0.6*x1[i] + 0.3*x2[i] + 0.2*x3[i]) + z0[i]*b3[k])
    dati$mu4[i] = exp((0.6*x1[i] + 0.3*x2[i] + 0.2*x3[i]) + z0[i]*b4[k])
    dati$mu5[i] = exp((0.3*x1[i] +0.15*x2[i] + 0.1*x3[i]) + z0[i]*b5[k])
    dati$mu6[i] = exp((0.3*x1[i] +0.15*x2[i] + 0.1*x3[i]) + z0[i]*b6[k])
    dati$mu7[i] = exp((0.6*x1[i] +0.3*x2[i] + 0.2*x3[i]) + z0[i]*b7[k,1] + z1[i]*b7[k,2])
    dati$mu8[i] = exp((0.6*x1[i] +0.3*x2[i] + 0.2*x3[i]) + z0[i]*b8[k,1] + z1[i]*b8[k,2])
    dati$mu9[i] = exp((0.3*x1[i] +0.15*x2[i] + 0.1*x3[i]) + z0[i]*b9[k,1] + z1[i]*b9[k,2])
    dati$mu10[i] = exp((0.3*x1[i] +0.15*x2[i] + 0.1*x3[i]) + z0[i]*b10[k,1] + z1[i]*b10[k,2])
    # simulate from a poisson with lambda = mu_ij
    dati$y1[i] = rpois(lambda=dati$mu1[i],  n=1)
    dati$y2[i] = rpois(lambda=dati$mu2[i],  n=1)
    dati$y3[i] = rpois(lambda=dati$mu3[i],  n=1)
    dati$y4[i] = rpois(lambda=dati$mu4[i],  n=1)
    dati$y5[i] = rpois(lambda=dati$mu5[i],  n=1)
    dati$y6[i] = rpois(lambda=dati$mu6[i],  n=1)
    dati$y7[i] = rpois(lambda=dati$mu7[i],  n=1)
    dati$y8[i] = rpois(lambda=dati$mu8[i],  n=1)
    dati$y9[i] = rpois(lambda=dati$mu9[i],  n=1)
    dati$y10[i] = rpois(lambda=dati$mu10[i], n=1)
  }
  
  # par(mfrow=c(5,2))
  # hist(dati$y1)
  # hist(dati$y2)
  # hist(dati$y3)
  # hist(dati$y4)
  # hist(dati$y5)
  # hist(dati$y6)
  # hist(dati$y7)
  # hist(dati$y8)
  # hist(dati$y9)
  # hist(dati$y10)
  
  
  
  B = data.frame(cluster=1:K, b1,b2,b3,b4,b5,b6,b7=b7,b8=b8,b9=b9,b10 =b10)
  
  result <- list(dati = dati, B = B)
  
  return(result)
}
