#################################################################################################
# Simulations for the 10 DGPs considering a binary response variable and non-linear fixed effects


#rm(list=ls())


require(rpart)
require(rpart.plot)
require(REEMtree)
require(nlme)
require(lme4)
require(boot)
require(MASS)
require(reshape)
require(plyr)
require(glmertree)
require(pre)

source("genera.dati.binomial.R")
source("find.cutoff.R")
source("mymixedtree.R")
source("GMERT.R")
source("extra_glmertree.R")
source("Speiser_BiMM_Tree.R")

maxiter = 100

PMAD_tree = matrix(rep(0,maxiter*10), nrow=maxiter, ncol=10)
PMAD_myI <- PMCR_myI  <- PMCR_myS <- PMAD_myS <- PMCR_tree <- PMAD_glm <- PMCR_glm  <- PMAD_tree
PMCR_glmm <- PMAD_glmm <- PMAD_gmert <- PMCR_gmert <- PMCR_glmert <- PMAD_glmert <- PMAD_BiMM <- PMCR_BiMM <- PMAD_tree


pb <- txtProgressBar(min = 0, max = maxiter*10, style = 3) #text based bar


Predictors <- "x1+x2+x3+x4+x5+x6+x7+x8"

random = rep(0,10)
random2 = rep(0,10)
random[1:6] =  '~  (1|cluster)'
random[7:10] =  '~  (1+z1|cluster)'
random2[1:6] =  ' (1|cluster)'
random2[7:10] =  ' ((1+z1)|cluster)'
RI <- ~  (1|cluster)
RIS <- ~  (1+z1|cluster)

risultati_PMAD = list()
risultati_PMCR = list()
###########################
##### SIMULAZIONE N.2  ####
###########################
set.seed(1993)

for(ITER in 1:maxiter) {
  
  n.train = 10
  n.test = 50
  K = 50
  N.test = K*n.test
  
  sim = genera.dati.non.lin(n.train,n.test,K)
  dati = sim$dati
  
  
  for(DGP in 1:10){
    
    Target = paste('y',DGP, sep='')
    F.tree = formula(paste(c(Target, Predictors), collapse = "~"))
    mu = paste('mu',DGP, sep='')
    mutrue = subset(dati[,mu], !dati$is.train)
    ytrue = subset(dati[,Target], !dati$is.train)
    mean.class = mean(subset(dati[,Target], dati$is.train))
    
    ## REGRESSION TREE
    tree = rpart(F.tree, data = dati, subset = is.train,
                 method = "class" )
    min.index <- which.min(tree$cptable[,"xerror"])
    prune_tree = prune.rpart(tree,cp=tree$cptable[min.index,"CP"])
    tree = prune_tree
    mupred = as.data.frame(predict(tree, newdata = subset(dati, !dati$is.train)))
    mupred = mupred$`1`
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_tree[ITER,DGP] = pmad*100
    ypred = predict(tree, newdata = subset(dati, !dati$is.train), type = "class")
    ypred = as.integer(ypred) -1
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_tree[ITER,DGP] = pmcr*100

    print(paste("tree", DGP, ITER)) 
    
    
    ## Mixed Tree, Random Intercept
    my = mymixedtree(F.tree, dataset = dati, random = RI,
                     subset = "is.train", family = binomial, maxdepth=10)
    mupred = as.data.frame(predict(my, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myI[ITER,DGP] = pmad*100
    pred.train = predict(my, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myI[ITER,DGP] = pmcr*100

    print(paste("myI", DGP, ITER))
    
    
    
    ## Mixed Tree, Random Intercept&Slope
    myS = mymixedtree(F.tree, dataset = dati, random = RIS,
                      subset = "is.train", maxdepth=10, family = binomial)
    mupred = as.data.frame(predict(myS, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myS[ITER,DGP] = pmad*100
    pred.train = predict(myS, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myS[ITER,DGP] = pmcr*100

    print(paste("myS", DGP, ITER))
    
    
    
    ## GLMM with known random structure and covariates x
    fixed = paste(c(Target, Predictors), collapse = "~")
    rand = formula(random[DGP])
    F.glmm = formula(paste(c(fixed, rand[[2]]), collapse = "+"))
    glm2 = glmer(F.glmm, data = dati, subset = is.train,
                family = "binomial" )
    mupred = as.data.frame(predict(glm2, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glmm[ITER,DGP] = pmad*100
    pred.train = predict(glm2, type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glmm[ITER,DGP] = pmcr*100
    
    
    print(paste("glmm", DGP, ITER))
    
    ## Fokkema glmertree
    formula_glmertree = paste(c("Target","~",Predictors,"|", random2[DGP],"|", Predictors ), collapse='')
    formula_glmertree = paste(c(Target,"~",1,"|", random2[DGP],"|", Predictors ), collapse='')

    dati_fok = dati[dati$is.train==T,]
    glmert = glmertree(formula_glmertree, data = dati_fok, family='binomial', maxit=30)
    mupred = as.data.frame(predict(glmert, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glmert[ITER,DGP] = pmad*100
    pred.train = predict(glmert, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glmert[ITER,DGP] = pmcr*100

    print(paste("glmert", DGP, ITER))

    
    
    # GMERT
    dati_gmert=subset(dati, dati$is.train)
    test_gmert = subset(dati, !dati$is.train)
    xnam = c("x1","x2","x3","x4","x5","x6","x7","x8")
    ni = rep(10,50)
    tni = rep(50,50)
    Xi = list()
    tXi = list()
    Zi = list()
    tZi = list()
    Yi = list()
    tYi = list()
    for(i in 1:50){
      Xi[[i]] = dati_gmert[dati_gmert$cluster==i,3:10]
      tXi[[i]] = test_gmert[test_gmert$cluster==i,3:10]
      if(random[DGP]=='~  (1|cluster)') {
        Zi[[i]]= as.matrix(rep(1,ni[i]))
        tZi[[i]]= as.matrix(rep(1,tni[i]))
        q=1
      }
      else if(random[DGP]=='~  (1+z1|cluster)') {
        Zi[[i]]= as.matrix(cbind(rep(1,ni[i]),dati_gmert$z1[dati_gmert$cluster==i]))
        tZi[[i]]= as.matrix(cbind(rep(1,tni[i]),test_gmert$z1[test_gmert$cluster==i]))
        q=2
      }
      Yi[[i]] = as.matrix(dati_gmert[dati_gmert$cluster==i,Target])
      tYi[[i]] = as.matrix(test_gmert[test_gmert$cluster==i, Target])
    }

    bizero <- list()
    length(bizero)<- 50
    for(i in 1:50)bizero[[i]] <- matrix(0,nrow=q,ncol=1)

    gmert_mod = gmert(ni,  Xi=Xi, Zi=Zi,  Yi=Yi, tni=tni, tXi=tXi, tZi=tZi, tYi=tYi, xnam=xnam ,usesurrogate=2
                      ,surrogatestyle=0,maxsurrogate=0,maxcompete=2,minsplit=30,minbucket=10,maxdepth=6,xval=10
                      ,cp=0
                      ,F=1			#use F=50 to reproduce results in HAJJEM, et al.  # Chiara: micro-iter mi sembrano F*4
                      ,s.change=1e-6
                      ,mert.niter= 10		#use mert.niter=200 to reproduce results in HAJJEM, et al.
                      ,L=1				#use L=50  to reproduce results in HAJJEM, et al.
                      ,mert.verbose = TRUE ,sigmasqzero=1,Dzero=diag(x = 0.01 ,nrow=q,ncol=q),bizero=bizero	,family=binomial
                      ,rc= 1e-4,gmert.niter = 1
                      ,S=1			#use S=55 to reproduce results in HAJJEM, et al.
                      ,verbose = TRUE)

    res_gmert = gmert_mod[[1]]
    mupred = as.data.frame(res_gmert$gmert.t.c.prob)
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_gmert[ITER,DGP] = pmad*100
    pred.train = res_gmert$gmert.l.c.prob
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_gmert[ITER,DGP] = pmcr*100

    print(paste("gmert", DGP, ITER))

    # BiMM
    
    dati_bimm = subset(dati, dati$is.train)
    ybimm = as.factor(dati_bimm[,Target])
    dati_bimm=data.frame(dati_bimm,ybimm)

    bimm_mod = bimmTree(formula = ybimm ~ x1+x2+x3+x4+x5+x6+x7+x8 , random= dati_bimm$cluster,
                        traindata=dati_bimm)

    mupred = predict(bimm_mod$Tree, newdata = subset(dati, !dati$is.train), type = "prob")[,2]
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_BiMM[ITER,DGP] = pmad*100
    pred.train = as.numeric(paste(predict(bimm_mod$Tree, newdata = subset(dati, dati$is.train), type = "class")))-1
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_BiMM[ITER,DGP] = pmcr*100

    print(paste("bimm", DGP, ITER))
    
  
    Sys.sleep(0.0001) 
    setTxtProgressBar(pb, (ITER-1)*10+DGP)
    
  }
  
}
close(pb)
print("******PMAD******")
round(colMeans(PMAD_myS), 3)
round(colMeans(PMAD_myI), 3)
round(colMeans(PMAD_tree), 3)
#round(colMeans(PMAD_glm), 3)
round(colMeans(PMAD_glmm), 3)
round(colMeans(PMAD_glmert), 3)
round(colMeans(PMAD_gmert), 3)

print("******PMCR******")
round(colMeans(PMCR_myS), 3)
round(colMeans(PMCR_myI), 3)
round(colMeans(PMCR_tree), 3)
#round(colMeans(PMCR_glm), 3)
round(colMeans(PMCR_glmm), 3)
round(colMeans(PMCR_glmert), 3)
round(colMeans(PMCR_gmert), 3)

