#################################################################################################
# Simulations for the 10 DGPs considering a binary response variable and tree-shaped fixed effects

#rm(list=ls())

require(rpart)
require(rpart.plot)
require(REEMtree)
require(nlme)
require(lme4)
require(boot)
require(MASS)
require(reshape)
require(plyr)
require(glmertree)
require(pre)

source("genera.dati.binomial.R")
source("find.cutoff.R")
source("mymixedtree.R")
source("GMERT.R")
source("extra_glmertree.R")
source("Speiser BiMM Tree.R")

maxiter = 100

PMAD_tree = matrix(rep(0,maxiter*10), nrow=maxiter, ncol=10)
PMAD_myI <- PMCR_myI  <- PMCR_myS <- PMAD_myS <- PMCR_tree <- PMAD_glm <- PMCR_glm  <- PMAD_tree
PMCR_glmm <- PMAD_glmm <- PMAD_gmert <- PMCR_gmert <- PMCR_glmert <- PMAD_glmert <- PMAD_BiMM <- PMCR_BiMM <- PMAD_tree

# Ability to recover the right tree structure
# 3 points: 1. right number of leaves; 2. right structure and right splitting covariates; 
#           3. right structure, right splitting covariates and right splitting cut points (within 1 from the real 1 that is 5)

NLEAVES_tree = matrix(rep(NA,maxiter*10), nrow=maxiter, ncol=10)
NSPLIT_tree = matrix(rep(NA,maxiter*10), nrow=maxiter, ncol=10)
NCUTP_tree = matrix(rep(NA,maxiter*10), nrow=maxiter, ncol=10)

NLEAVES_myI <- NLEAVES_myS <- NLEAVES_gmert <- NLEAVES_glmert <- NLEAVES_BiMM <- NLEAVES_tree
NSPLIT_myI <- NSPLIT_myS <- NSPLIT_gmert <- NSPLIT_glmert <- NSPLIT_BiMM <-NSPLIT_tree
NCUTP_myI <- NCUTP_myS <- NCUTP_gmert <- NCUTP_glmert <- NCUTP_BiMM <- NCUTP_tree

pb <- txtProgressBar(min = 0, max = maxiter*10, style = 3) #text based bar


Predictors <- "x1+x2+x3+x4+x5+x6+x7+x8"

random = rep(0,10)
random2 = rep(0,10)
random[1:6] =  '~  (1|cluster)'
random[7:10] =  '~  (1+z1|cluster)'
random2[1:6] =  ' (1|cluster)'
random2[7:10] =  ' ((1+z1)|cluster)'
RI <- ~  (1|cluster)
RIS <- ~  (1+z1|cluster)

correct_struc=c("x1", "x2", "<leaf>", "x4",  "<leaf>", "<leaf>", "x3", "x5", "<leaf>", "<leaf>", "<leaf>")
###########################
##### SIMULAZIONE N.2  ####
###########################
set.seed(1993)

for(ITER in 1:maxiter) {
  
  n.train = 10
  n.test = 50
  K = 50
  N.test = K*n.test
  
  sim = genera.dati(n.train,n.test,K)
  dati = sim$dati
  
  
  for(DGP in 1:10){
    
    Target = paste('y',DGP, sep='')
    F.tree = formula(paste(c(Target, Predictors), collapse = "~"))
    mu = paste('mu',DGP, sep='')
    mutrue = subset(dati[,mu], !dati$is.train)
    ytrue = subset(dati[,Target], !dati$is.train)
    mean.class = mean(subset(dati[,Target], dati$is.train))
    
    ## REGRESSION TREE
    tree = rpart(F.tree, data = dati, subset = is.train,
                 method = "class" )
    min.index <- which.min(tree$cptable[,"xerror"])
    prune_tree = prune.rpart(tree,cp=tree$cptable[min.index,"CP"])
    tree = prune_tree
    mupred = as.data.frame(predict(tree, newdata = subset(dati, !dati$is.train)))
    mupred = mupred$`1`
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_tree[ITER,DGP] = pmad*100
    ypred = predict(tree, newdata = subset(dati, !dati$is.train), type = "class")
    ypred = as.integer(ypred) -1
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_tree[ITER,DGP] = pmcr*100

    NLEAVES_tree[ITER,DGP] = dim(table(tree$where))
    if(!is.na(NLEAVES_tree[ITER,DGP])){
      if(NLEAVES_tree[ITER,DGP]==6) NSPLIT_tree[ITER,DGP] = (sum(sort(tree$frame$var) != sort(correct_struc))==0)
    }
    if(!is.na(NSPLIT_tree[ITER,DGP])){
      if(NSPLIT_tree[ITER,DGP]==T) {
        splits = rep(0,10)
        for(i in 2:11){
          splits[i-1]=as.numeric(paste(substring(labels(tree)[i],5,9)))
        }
        splitf = unique(splits)

        NCUTP_tree[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
      }
    }
    
    
    
    ## GLMM with known structure
    fixed = paste(c(Target, "as.factor(leaf)"), collapse = "~")
    rand = formula(random[DGP])
    F.glmm = formula(paste(c(fixed, rand[[2]]), collapse = "+"))
    glm = glmer(F.glmm, data = dati, subset = is.train,
                family = "binomial" )
    mupred = as.data.frame(predict(glm, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glm[ITER,DGP] = pmad*100
    pred.train = predict(glm, type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glm[ITER,DGP] = pmcr*100
    
    
    
    ## Mixed Tree, Random Intercept
    my = mymixedtree(F.tree, dataset = dati, random = RI,
                     subset = "is.train", family = binomial, maxdepth=4)
    mupred = as.data.frame(predict(my, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myI[ITER,DGP] = pmad*100
    pred.train = predict(my, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myI[ITER,DGP] = pmcr*100

    NLEAVES_myI[ITER,DGP] = dim(table(my$Tree$where))
    if(!is.na(NLEAVES_myI[ITER,DGP])){
      if(NLEAVES_myI[ITER,DGP]==6) NSPLIT_myI[ITER,DGP] = (sum(sort(my$Tree$frame$var) != sort(correct_struc))==0)
    }
    if(!is.na(NSPLIT_myI[ITER,DGP])){
      if(NSPLIT_myI[ITER,DGP]==T) {
        splits = rep(0,10)
        for(i in 2:11){
          splits[i-1]=as.numeric(paste(substring(labels(my$Tree)[i],5,9)))
        }
        splitf = unique(splits)

        NCUTP_myI[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
      }
    }
    
    
    
    ## Mixed Tree, Random Intercept&Slope
    myS = mymixedtree(F.tree, dataset = dati, random = RIS,
                      subset = "is.train", maxdepth=4, family = binomial)
    mupred = as.data.frame(predict(myS, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_myS[ITER,DGP] = pmad*100
    pred.train = predict(myS, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_myS[ITER,DGP] = pmcr*100

    NLEAVES_myS[ITER,DGP] = dim(table(myS$Tree$where))
    if(!is.na(NLEAVES_myS[ITER,DGP])){
      if(NLEAVES_myS[ITER,DGP]==6) NSPLIT_myS[ITER,DGP] = (sum(sort(myS$Tree$frame$var) != sort(correct_struc))==0)
    }
    if(!is.na(NSPLIT_myS[ITER,DGP])){
      if(NSPLIT_myS[ITER,DGP]==T) {
        splits = rep(0,10)
        for(i in 2:11){
          splits[i-1]=as.numeric(paste(substring(labels(myS$Tree)[i],5,9)))
        }
        splitf = unique(splits)

        NCUTP_myS[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
      }
    }
    
    
    
    ## GLMM with known structure and covariates x
    fixed = paste(c(Target, Predictors), collapse = "~")
    rand = formula(random[DGP])
    F.glmm = formula(paste(c(fixed, rand[[2]]), collapse = "+"))
    glm2 = glmer(F.glmm, data = dati, subset = is.train,
                family = "binomial" )
    mupred = as.data.frame(predict(glm2, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glmm[ITER,DGP] = pmad*100
    pred.train = predict(glm2, type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glmm[ITER,DGP] = pmcr*100
    
    
    
    
    ## Fokkema glmertree
    formula_glmertree = paste(c("Target","~",Predictors,"|", random2[DGP],"|", Predictors ), collapse='')
    formula_glmertree = paste(c(Target,"~",1,"|", random2[DGP],"|", Predictors ), collapse='')

    dati_fok = dati[dati$is.train==T,]
    glmert = glmertree(formula_glmertree, data = dati_fok, family='binomial')
    mupred = as.data.frame(predict(glmert, newdata = subset(dati, !dati$is.train), type = "response"))
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_glmert[ITER,DGP] = pmad*100
    pred.train = predict(glmert, newdata = subset(dati, dati$is.train), type = "response")
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_glmert[ITER,DGP] = pmcr*100

    NLEAVES_glmert[ITER,DGP] = length(coef(glmert$tree))

    if(NLEAVES_glmert[ITER,DGP] ==6){
    paths = list.paths(glmert$tree) # lists paths to all terminal nodes
    splits <- list.splits(paths)

    splitf = rep(0,length(splits)[[1]])
    for(i in 1:length(splits)[[1]]){
      if(substring((splits[i]),8,8)=='.') splitf[i]=as.numeric(paste(substring(splits[i],7,11)))
      else if(substring((splits[i]),7,7)=='.') splitf[i]=as.numeric(paste(substring(splits[i],6,10)))
    }
    
    splitf = unique(splitf)
    NSPLIT_glmert[ITER,DGP] = (length(splitf)==5)

    if(NSPLIT_glmert[ITER,DGP]==T)    NCUTP_glmert[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
}
    
    # plot(glmert$tree, type = "simple")
    # plot(glmert$tree, which = "tree", fitted = "combined")
    # plot(glmert$tree, which = "tree", fitted = "none")
    # plot(glmert$tree, which = "tree", observed = FALSE)
    # plot(glmert$tree, which = "tree.coef")
    # plot(glmert$tree, which = "ranef")
    
    
    # GMERT
    dati_gmert=subset(dati, dati$is.train)
    test_gmert = subset(dati, !dati$is.train)
    xnam = c("x1","x2","x3","x4","x5","x6","x7","x8")
    ni = rep(10,50)
    tni = rep(50,50)
    Xi = list()
    tXi = list()
    Zi = list()
    tZi = list()
    Yi = list()
    tYi = list()
    for(i in 1:50){
      Xi[[i]] = dati_gmert[dati_gmert$cluster==i,3:10]
      tXi[[i]] = test_gmert[test_gmert$cluster==i,3:10]
      if(random[DGP]=='~  (1|cluster)') {
        Zi[[i]]= as.matrix(rep(1,ni[i]))
        tZi[[i]]= as.matrix(rep(1,tni[i]))
        q=1
      }
      else if(random[DGP]=='~  (1+z1|cluster)') {
        Zi[[i]]= as.matrix(cbind(rep(1,ni[i]),dati_gmert$z1[dati_gmert$cluster==i]))
        tZi[[i]]= as.matrix(cbind(rep(1,tni[i]),test_gmert$z1[test_gmert$cluster==i]))
        q=2
      }
      Yi[[i]] = as.matrix(dati_gmert[dati_gmert$cluster==i,Target])
      tYi[[i]] = as.matrix(test_gmert[test_gmert$cluster==i, Target])
    }

    bizero <- list()
    length(bizero)<- 50
    for(i in 1:50)bizero[[i]] <- matrix(0,nrow=q,ncol=1)

    gmert_mod = gmert(ni,  Xi=Xi, Zi=Zi,  Yi=Yi, tni=tni, tXi=tXi, tZi=tZi, tYi=tYi, xnam=xnam ,usesurrogate=2
                      ,surrogatestyle=0,maxsurrogate=0,maxcompete=2,minsplit=30,minbucket=10,maxdepth=6,xval=10
                      ,cp=0
                      ,F=1			#use F=50 to reproduce results in HAJJEM, et al.  # Chiara: micro-iter mi sembrano F*4
                      ,s.change=1e-6
                      ,mert.niter= 10		#use mert.niter=200 to reproduce results in HAJJEM, et al.
                      ,L=1				#use L=50  to reproduce results in HAJJEM, et al.
                      ,mert.verbose = TRUE ,sigmasqzero=1,Dzero=diag(x = 0.01 ,nrow=q,ncol=q),bizero=bizero	,family=binomial
                      ,rc= 1e-4,gmert.niter = 1
                      ,S=1			#use S=55 to reproduce results in HAJJEM, et al.
                      ,verbose = TRUE)

    res_gmert = gmert_mod[[1]]
    mupred = as.data.frame(res_gmert$gmert.t.c.prob)
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_gmert[ITER,DGP] = pmad*100
    pred.train = res_gmert$gmert.l.c.prob
    p0 = find.cutoff(pred.train, mean.class)
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_gmert[ITER,DGP] = pmcr*100



    NLEAVES_gmert[ITER,DGP] = dim(table(gmert_mod[[1]]$gmert.tree$where))
    if(!is.na(NLEAVES_gmert[ITER,DGP])){
      #if(NLEAVES_gmert[ITER,DGP]==6) NSPLIT_glmert[ITER,DGP] = (sum(gmert_mod[[1]]$gmert.tree$frame$var != correct_struc)==0)
      if(NLEAVES_gmert[ITER,DGP]==6) NSPLIT_glmert[ITER,DGP] = (sum(sort(gmert_mod[[1]]$gmert.tree$frame$var) != sort(correct_struc))==0)
    }
    if(!is.na(NSPLIT_gmert[ITER,DGP])){
      if(NSPLIT_gmert[ITER,DGP]==T) {
        splits = rep(0,10)
        for(i in 2:11){
          splits[i-1]=as.numeric(paste(substring(labels(gmert_mod[[1]]$gmert.tree)[i],5,9)))
        }
        splitf = unique(splits)

        NCUTP_gmert[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
      }
    }
    
    # BiMM
    
    dati_bimm = subset(dati, dati$is.train)
    ybimm = as.factor(dati_bimm[,Target])
    dati_bimm=data.frame(dati_bimm,ybimm)
    
    bimm_mod = bimmTree(formula = ybimm ~ x1+x2+x3+x4+x5+x6+x7+x8 , random= dati_bimm$cluster, 
                        traindata=dati_bimm)
    
    mupred = predict(bimm_mod$Tree, newdata = subset(dati, !dati$is.train), type = "prob")[,2]
    pmad = 1/N.test * sum(abs(mupred-mutrue))
    PMAD_BiMM[ITER,DGP] = pmad*100
    pred.train = as.numeric(paste(predict(bimm_mod$Tree, newdata = subset(dati, dati$is.train), type = "class")))
    #p0 = find.cutoff(pred.train, mean.class)
    p0=0.5
    ypred = as.integer(mupred>p0)
    pmcr = 1/N.test * sum(abs(ypred- ytrue))
    PMCR_BiMM[ITER,DGP] = pmcr*100
    
    #rpart.plot(bimm_mod$Tree)
    #ranef(bimm_mod$EffectModel)
    
    NLEAVES_BiMM[ITER,DGP] = dim(table(bimm_mod$Tree$where))
    if(!is.na(NLEAVES_BiMM[ITER,DGP])){
      if(NLEAVES_BiMM[ITER,DGP]==6) NSPLIT_BiMM[ITER,DGP] = (sum(sort(bimm_mod$Tree$frame$var) != sort(correct_struc))==0)
    }
    if(!is.na(NSPLIT_BiMM[ITER,DGP])){
      if(NSPLIT_BiMM[ITER,DGP]==T) {
        splits = rep(0,10)
        for(i in 2:11){
          splits[i-1]=as.numeric(paste(substring(labels(bimm_mod$Tree)[i],5,9)))
        }
        splitf = unique(splits)
        
        NCUTP_BiMM[ITER,DGP] = (sum((splitf>4)&(splitf<6)))
      }
    }
    
  
    Sys.sleep(0.0001) 
    setTxtProgressBar(pb, (ITER-1)*10+DGP)
    
  }
  print(ITER)
}
close(pb)
print("******PMAD******")
round(colMeans(PMAD_myS), 3)
round(colMeans(PMAD_myI), 3)
round(colMeans(PMAD_tree), 3)
round(colMeans(PMAD_glm), 3)
round(colMeans(PMAD_glmm), 3)
round(colMeans(PMAD_glmert), 3)
round(colMeans(PMAD_gmert), 3)

print("******PMCR******")
round(colMeans(PMCR_myS), 3)
round(colMeans(PMCR_myI), 3)
round(colMeans(PMCR_tree), 3)
round(colMeans(PMCR_glm), 3)
round(colMeans(PMCR_glmm), 3)
round(colMeans(PMCR_glmert), 3)
round(colMeans(PMCR_gmert), 3)

